# Import sample data.  This import structure was taken directly from skimages
# data import style:
    # https://github.com/scikit-image/scikit-image

import os.path as op
from skspec import data_dir, TimeSpectra, Spectra
from skspec.core.specindex import SpecIndex
from skspec.units import Unit
from skspec.pandas_utils.dataframeserial import df_load
from pandas import read_csv

__all__ = ['aunps_glass', 'aunps_water', 'solvent_evap', 'trip_peaks']

class DataError(Exception):
    """ """

def load_ts(filepath, *args, **kwargs):
    """
    Parameters
    ----------
    f : string
        File name of .csv or .pickle spectrum.

    Returns
    -------
    TimeSpectra
    """
    
    filepath = op.join(data_dir, filepath)
    ext = op.splitext(filepath)[1]
    

    if ext == '.csv':
        return TimeSpectra.from_csv(filepath, *args, **kwargs)

    #  XXX INCOMPLETE FUNCTIONALITY
    elif ext == '.pickle':
        df = df_load(filepath)
        logger.critical("LOADING FROM PICKLE NOT COMPLETE YET")

    else:
        raise DataError('%s must have file extension .csv or .pickle, not '
                             '%s' %(filepath, ext))
 

def _load_gwuspec(filepath, *args, **kwargs):
    """ Loads GWU data from csv, assigns baseline and crops accordingly.
    This is a wrapper to let author use his own data for the example data."""
    
    # CSV KWARGS
    kwargs.setdefault('index_col', 0)
    kwargs.setdefault('header_datetime', "%Y-%m-%d %H:%M:%S")    

    # TimeSpec KWARGS
    kwargs.setdefault('reference', 0) #Index col will be removed.
    kwargs.setdefault('specunit', 'nm')
        
    ts = load_ts(filepath, *args, **kwargs)

    # Baselines removed apriori with dynamic_baseline
    return ts


def trip_peaks(*args, **kwargs):
    """ Sample data packaged from 2d Shiege:
        (http://sci-tech.ksc.kwansei.ac.jp/~ozaki/2D-shige.htm)
    Data depicts three changing peaks.  Perturbation units is genera
    """
    
    filepath = op.join(data_dir, 'triple_peaks.csv')
    trips = Spectra.from_csv(filepath,
                             header=None, #Just use intindex, overwrite anyway
                             name='Three Peaks',
                             index_col=0,
                             skiprows=1)   
    trips.reference = 0
   
    # Hack because can't do floating point on SpecIndex (should change csv file)
    trips.index = SpecIndex(trips.index+50, unit='ev')
    # Generalized perturbation Unit, making it up
    trips.varunit = Unit(short='pz', full='Polarization', symbol=r'\phi')
    
    return trips
    


def solvent_evap(*args, **kwargs):
    """ Model solvent evaporation dataset graciously shared by Dr. Isao Noda;
    used in his 2004 book Two-Dimensional Correlation Spectroscopy.  From 
    page 47:
    
    'The system described here is a three-component solution mixture of 
    polystyrene (PS) dissolved in a 50:50 blblend of metyl etyl ketone (MEK) and
    perdeuterated tolune.  The initial concentration of PS is about 1.0wt%.
    Once the solution mixture is exposed to the open atmosphere, the solvents 
    start evaporating, and the PS concentraiton increases with time.  However,
    due to the substantial difference in the volatility of MEK and toluene 
    coupled with their slightly dissimilar affinity to PS, the composition of
    the solution mixture changes as a function of time in a rather complex 
    manner during hte spontaneous evaporation process.  
    
    The transient IR spectra were collected as the two solvents evaporated, 
    eventually leaving a PS film behind, as shown schematically in Figure 4.1
    (A).  The measurement was actually made using a horizontal attenuated total 
    reflectance (ATR) prism.  
    
    ...
    
    As expected, the intensities of bands at 2980 and 1720 cm-1 due to violatile
    MEK and those of bands at 2275 and 820cm-1 assigned to perdeuterated toluene
    gradually decrease, while those of PS bands at 3020 and 1450cm-1 increase 
    with time.'
    
    """
    
    # CSV KWARGS
    kwargs.setdefault('index_col', 0)

    # TimeSpec KWARGS
    kwargs.setdefault('reference', 0) #Index col will be removed.
    kwargs.setdefault('specunit', 'cm-1')
    kwargs.setdefault('name', 'Polystyrene Evaporation')    
    kwargs.setdefault('varunit', 'm')

#    kwargs.setdefault('dtype', 'float64') #For parser (data not labels)

    ts = load_ts('pskdt.csv', *args, **kwargs)    
    return ts


def aunps_glass(*args, **kwargs):
    """ Reeveslab data obtained by Adam Hughes of gold nanoparticles (AuNPs)
    on a glass optical fiber; spectrum corresponds to the specular Reflectance.
    Please reference skspec if citing.  

    Parameters
    ----------
    style : int (1,2,or 3
        represents three types of AuNPs on fibers.  style 1 is very homegenous
        coverage.  style 2 is fair coverage with early signs of aggregation.
        style=3 is poor coverage is many AuNP aggregates.
   
    Notes
    -----
    style 1 correponds to dataset 5/22/14, fiber 4.  style 2 from 9/11/13 fiber 1
    and style 3 from 10/1/DTSSP fiber 1.  All of these were cropped and resampled
    to have exatly 100 columns.  Such sampling may exagerate features of the 
    original dataset, such as the temporal gaps in the style=3 dataset.  They were
    there in original data, but sampling makes them seem more pronounced or intense.
    """
    style = kwargs.pop('style', 1)
    
    if style == 1 or style == 'good':
        kwargs.setdefault('name', 'AuNPs Glass (good)')    
        return _load_gwuspec('aunps_glass_good.csv', *args, **kwargs)
        

    elif style == 2 or style == 'fair':
        kwargs.setdefault('name', 'AuNPs Glass (fair)')    
        return _load_gwuspec('aunps_glass_fair.csv', *args, **kwargs)


    elif style == 3 or style == 'poor':
        kwargs.setdefault('name', 'AuNPs Glass (poor)')   
        return _load_gwuspec('aunps_glass_bad.csv', *args, **kwargs)
    
    elif style == 4 or style == 'full':
        kwargs.setdefault('name', 'AuNPs Glass (full)')   
        return _load_gwuspec('aunps_glass_full.csv', *args, **kwargs)

    else:
        raise DataError("style argument must be '1' or 'good'; '2' or 'fair';"
            " '3' or 'poor'; '4' or 'full'; instead got '%s'" % style)

    

def aunps_water(*args, **kwargs):
    """ Reeveslab data obtained by Adam Hughes of gold nanoparticles in water. 
    UV peak is due to residual citrates from citrate reduction in synthesizing
    the nanoparticles via the Turkevich method.
    Please reference skspec if citing.  Added baseline = 234.0 array.
    """
    kwargs.setdefault('name', 'Gold Nanoparticles in Water')    
    ts = _load_gwuspec('aunps_water.csv', *args, **kwargs)
    ts.baseline = 234.0
    return ts
    
if __name__ == '__main__':
    ts = aunps_water()
    print ts, type(ts)
    



_DESCRIPTION_CHARS = 40 #Max number of characters to format in description

class RegisterError(Exception):
    """ """

class PlotInfo(object):
    """Stores key, function, metadata and description for plot"""

    def __init__(self, key, function, is_3d, description=None):
            
        self.key = key
        self.function = function

        # Track module for each function
        self._module = function.__module__
        
        if not isinstance(is_3d, bool):
            raise AttributeError('is_3d must be True or False, got %s' % is_3d)
        self.is_3d = is_3d

        if not description:
            description = function.__doc__        
        self.description = description
        
    def __repr__(self):
        description = self.description
        if len(description) > _DESCRIPTION_CHARS:
            description = description[0:10] + '...'
        return '\t'.join([str(self.key),
                          self.function.__name__ + '()', 
                          str(self.is_3d), 
                          description])

    def __shortrepr__(self):
        """ Short representaiton, substitue for __repr__() """
        description = self.description
        if len(description) > _DESCRIPTION_CHARS:
            description = description[0:10] + '...'
        return '\t'.join([str(self.key),description])


class PlotRegister(object):
    """ Basic storage class to track all various plot types in skspec;
    simple relational mapper for operations like "return all 3D plots.
    
    Notes:
    ------
    
    Need to be able to parse plots in Spectra.plot() and also in widget notebooks.
    As plotting gets more involved, this will become more useful.
    """
    
    def __init__(self, plots={}):
        for p in plots:
            if not isinstance(PlotInfo):
                raise RegisterError('All plots in registry must be PlotInfo object')
        self.plotdict = dict((p.key, p) for p in plots)

    @property
    def plots(self):
        plots = self.plotdict.values()
        return sorted(plots, key=lambda x: x.key)
        
    @property
    def keys(self):
        return sorted(self.plotdict.keys()) 

    @property
    def functions(self):
        return sorted([p.function for p in self.plots])
    
    #@property
    #def is_3d(self):
        #return sorted([p.is_3d for p in self.plots]) 
    
    def is_3d(self, kind):
        """ Is this kind a 3d plot """
        return self.plotdict[kind].is_3d
    
    def is_2d(self, kind):
        """ Is this a 2d plot (contour)"""
        if kind in ['contour']:
            return True
        return False
    
    def is_2d_3d(self, kind):
        """ Is 3d or contour/other mesh plots?"""
        if self.is_2d(kind) or self.is_3d(kind):
            return True
        return False
    
    @property
    def descriptions(self):
        return sorted([p.description for p in self.plots])   
    
    def add(self, *args, **kwargs):
        """ Passed to PlotInfo"""
        pout = PlotInfo(*args, **kwargs)
        self.plotdict[pout.key] = pout   
    
    @property
    def plots_3d(self):
        """ Return plot names that are 3D; for created 3d axes mostly """
        out = []
        for p in self.plots:
            if p.is_3d:
                out.append(p.key)
        return out

    @property
    def plots_2d_3d(self):
        """ All 3d plots + contour"""
        return self.plots_3d + ['contour']
    
    #Dict Interface / Magic
    def __getitem__(self, key):
        return self.plotdict.__getitem__(key)
        
    def __delitem__(self, key):
        return self.plotdict.__delitem__(key)
    
    def __setitem__(self, key, value):
        return self.plotdict.__setitem__(key, value)    

    def items(self):
        return self.plotdict.items()

    def keys(self):
        return self.plotdict.keys()
    
    def values(self):
        return self.plotdict.values() 

    def __repr__(self):
        # MAKE ACTUAL COLUMNS
        out = '\t'.join(['KIND', 'FUNCTION', '3D', 'DESCRIPTION'])
        out += '\n------------------------------------------'
        out += '\n\n'
        out += '\n'.join([v.__repr__() for v in self.plots])
        return out    
    

    def __shortrepr__(self):
        # MAKE ACTUAL COLUMNS
        out = '\t'.join(['KIND', 'DESCRIPTION'])
        out += '\n----------------'
        out += '\n\n'
        out += '\n'.join([v.__shortrepr__() for v in self.plots])
        return out    
    



from skspec.plotting.advanced_plots import PLOTPARSER

from IPython.html.widgets import (
    FlexBox, VBox, HBox, HTML, Box, RadioButtons,
    Text, Dropdown, Checkbox, Image, Popup,
    IntSlider, Button, Text, FloatSlider, IntText, 
    ContainerWidget, FloatText
    )

import IPython.display as ipdisplay

from IPython.utils.traitlets import link, Unicode
from jinja2 import Template

class PopOvers(HTML):
    template = Template("""
        <p id="x"
        class="{{classname}}"
        title="{{ title|escape }}"
        data-toggle="popover"
        data-content="{{ content|escape }}">{{description}}</p>
        <script>
        $('#x').popover();
        </script>
        """)
    description = Unicode(sync=True)
    classname = Unicode(sync=True)
    title = Unicode(sync=True)
    content = Unicode(sync=True)


    def __init__(self, *args, **kwargs):
        super(PopOvers, self).__init__(*args, **kwargs)

    def _title_changed(self, old, new):
        self.value = self.template.render(**self._trait_values)


class PanelTitle(HTML):
    def __init__(self, *args, **kwargs):
        super(PanelTitle, self).__init__(*args, **kwargs)
        self._dom_classes += ("panel-heading panel-title",)

class PanelBody(Box):
    def __init__(self, *args, **kwargs):
        super(PanelBody, self).__init__(*args, **kwargs)
        self._dom_classes += ("panel-body",)

class ControlPanel(Box):
    # A set of related controls, with an optional title, in a box (provided by CSS)
    def __init__(self, title=None, *args, **kwargs):
        super(ControlPanel, self).__init__(*args, **kwargs)
        self._dom_classes += ("panel panel-info",)

        # add an option title widget
        if title is not None:            
            self.children = [
                PanelTitle(value=title),
                PanelBody(children=self.children)
            ]


class Alert(HTML):
    """HTML widget that is used to store alerts.  For now,
    just a pure HTML class but put in in separate class to
    allow potential customaization in future."""


class SpectraGui(Box):
    """
    An example GUI for a spectroscopy application.

    Note that `self.model` is the owner of all of the "real" data, while this
    class handles creating all of the GUI controls and links. This ensures
    that the model itself remains embeddable and rem
    """
    more = Checkbox(description="Advanced")

    def __init__(self, model=None, model_config=None, *args, **kwargs):
	# RUN HTML
        self.model = model or Spectrogram(**(model_config or {}))  #Need to access spectrogram if defaulting...
        # Create alert widget (refactor into its own function?)
        alert = Alert(description = "Alert or something")
        link((self.model, "message"), (alert, "value"))

        # Create a GUI
        kwargs["orientation"] = 'horizontal'
        kwargs["children"] = [
            HBox([VBox([alert, self.model]),
                  VBox([self.save_load_panel(), 
                        self.load_panel(),
                        self.unit_panel()]),
                       ]),
            self._controls(),
        ]
        super(SpectraGui, self).__init__(*args, **kwargs)
        self._dom_classes += ("spectroscopy row",)

    def _controls(self):
        panels = VBox([
            HBox([self.plot_panel(),self.slicing_panel()]),
            HBox([self.plot_plugin_panel(),self.user_function_panel()])],
                      _dom_classes=["col-xs-3"])

        return panels

    def set_layout(self):
	return ipdisplay.HTML(open('style1.html','r').read())

    def load_panel(self):
        # create correlation controls. NOTE: should only be called once.
        loadbutton = Button(description = 'Load')

        loaddata = Checkbox(description="Testdata")
        link((self.model, "load_spec"), (loaddata, "value"))
        testdataset = Text(description = "") 
        link((self.model, "testdataset"), (testdataset, "value"))
     
        specbox = HBox([loadbutton, testdataset])
        link((self.model, "load_spec"), (specbox, "visible"))

        loadfile = Checkbox(description="NB Variable") #Test Data
        link((self.model, "load_file"), (loadfile, "value"))
        filename = Text(description = "")

        link((self.model, "file_name"), (filename, "value"))
        filebox = HBox([loadbutton, filename])
        link((self.model, "load_file"), (filebox, "visible"))

        reset = Button(color='white',background_color='violet',
                       escription='Reset Defaults')
        #reset.on_click(lambda x: self.model.)

        return ControlPanel(title="Load Dataset", children=[HBox(children=[loaddata,loadfile]),specbox,filebox,reset])

    def plot_panel(self):
        # create draw mode controls.  NOTE: should only be called once.
        cbar = Checkbox(description="Colorbar")
        link((self.model, "colorbar"), (cbar, "value"))

        interact = Checkbox(description="Interactive")
        link((self.model, "interactive"), (interact, "value"))

        plug_select = Checkbox(description="Line Selection")
        link((self.model, "selectlines"), (plug_select, "value"))

        autoupdate = Checkbox(description="Auto Update")
        link((self.model, "autoupdate"), (autoupdate, "value"))

        plugin2= Checkbox(description='plugin2')
        plugin3= Checkbox(description='plugin3')
        fwidth = FloatText(description='Plot width')
        fheight = FloatText(description='Plot height')
        link((self.model, "figwidth"), (fwidth, "value"))
        link((self.model, "figheight"), (fheight, "value"))
        
        
        f = Text(description="Function:")
        link((self.model,"user_f"),(f,"value"))
        fapp = Button(description = "Apply")
        fapp.on_click(lambda x: self.model.apply_userf(name='apply clicked'))

        #plugins = HBox([plugin1,plugin2,plugin3])
        #more = Checkbox(description="More Options")### LINK IT
        #link((self, "moreopt"), (more, "value"))
        popmore = Popup(children=[VBox([HBox([plug_select,plugin2,plugin3]),
                                        HBox([f,fapp]),
                                        VBox([fwidth, fheight])
                                      ])],
                        description='Advanced', button_text='Advanced')

        cmap = Dropdown(description="Colormap",values=self.model.COLORMAPS)
        link((self.model,"colormap"),(cmap,"value"))

        color = Dropdown(description="Color",values=self.model.COLORS)
        link((self.model, "color"), (color, "value"))

        kind = Dropdown(description="Kind", values=PLOTPARSER.keys())
        link((self.model, "kind"), (kind, "value"))


        return ControlPanel(title="Plot Settings", 
                            children=[
                                HBox([autoupdate, interact, self.more]),
                                HBox([cmap,cbar]),
                                HBox([color, kind]),
                                popmore
                                ]
                            )

    def unit_panel(self):
        # create spectrum controls.  NOTE: should only be called once.
        specunit = Text(description="Specunit",values=self.model.spec_unit)
        link((self.model,"spec_unit"),(specunit,"value"))

        varunit = Text(description="Varunit",values=self.model.var_unit)
        link((self.model,"var_unit"),(varunit,"value"))

        iunit = Text(description="I Unit",values=self.model.iunit)
        link((self.model,"iunit"),(iunit,"value"))

        normunit = Dropdown(description="Normunit",values=self.model.NORMUNITS.values())
        link((self.model,"norm_unit"),(normunit,"value"))

        return ControlPanel(title="Units",
                            children= [
                                normunit,
                                specunit,
                                varunit,
                                iunit]
                            )

    def slicing_panel(self):
        """ create spectrum controls.  NOTE: should only be called once."""

        model = self.model #For readability

        # ALL WIDGETS ARE CAPITALIZED. 
        #AXIS = RadioButtons(values=[0,1],description="Axis")
        #link((model, "slice_axis"), (AXIS, "value"))

        SPECSTART = FloatSlider(description="Spec Slice Start",
                                min=model.specslice_position_start) #Will not try to update

        link((model,"specslice_position_start"),(SPECSTART,"value"))
        link((model,"specstep"),(SPECSTART,"step"))
        link((model,"specslider_start"),(SPECSTART,"min")) # Start end values (IE slider_min / slider_max)
        link((model,"specslider_end"),(SPECSTART,"max"))

        SPECEND = FloatSlider(description="Spec Slice End",
                              max=model.specslice_position_end) # Will not try to update

        link((model,"specslice_position_end"),(SPECEND,"value"))
        link((model,"specstep"),(SPECEND,"step"))
        link((model,"specslider_start"),(SPECEND,"min"))
        link((model,"specslider_end"),(SPECEND,"max"))

        # SPACING WIDGET
        SPECSPACING = IntText(description="Spec Sample by",value=1)
        link((model,"specspacing"),(SPECSPACING,"value"))

        TIMESTART = FloatSlider(description="Time Slice Start",
                                min=model.timeslice_position_start)

        link((model,"timeslice_position_start"),(TIMESTART,"value"))
        link((model,"timestep"),(TIMESTART,"step"))
        link((model,"timeslider_start"),(TIMESTART,"min"))
        link((model,"timeslider_end"),(TIMESTART,"max"))

        TIMEEND = FloatSlider(description="Time Slice End",
                              max=model.timeslice_position_end)
        link((model,"timeslice_position_end"),(TIMEEND,"value"))
        link((model,"timestep"),(TIMEEND,"step"))
        link((model,"timeslider_start"),(TIMEEND,"min"))
        link((model,"timeslider_end"),(TIMEEND,"max"))

        TIMESPACING = IntText(description="Time Sample by",value=1)
        link((model,"timespacing"),(TIMESPACING,"value"))

        RANGED = VBox([SPECSTART,SPECEND,SPECSPACING, TIMESTART, TIMEEND, TIMESPACING])


        return ControlPanel(title="Slicing/Sampling",
                            children=[
                                #AXIS,
                                RANGED
                            ]
                            )

    def save_load_panel(self):
        """ All Save/Load IO of GUI in this panel. """
        saveplot = Button(description='Save Plot')
        savets = Button(description='Export as NB Variable')
        #link((self.model,"save_spec"),(savets,'value'))
        savets.on_click(lambda x: self.model.save_to_ns())
        savetsas = Text(description='Save as:')
        link((self.model,"save_spec_as"),(savetsas,"value"))

        po= PopOvers(description='Click for Popover')
        link((self.model,"CONTENT"),(po,"content"))
        link((self.model,"classname"),(po,"classname"))
        link((self.model,"title"),(po,"title"))
        #redraw = Button(description="Redraw")
        #redraw.on_click(lambda x: self.model.draw())
        return ControlPanel(title='Save Dataset (combine w/ load)',
                            children=[saveplot,
                                      savets,
                                      savetsas,
                                      po])
        #return ToolBar(redraw)


    def plot_plugin_panel(self):
        plugin1= Checkbox(description='plugin1')
        plugin2= Checkbox(description='plugin2')
        plugin3= Checkbox(description='plugin3')
        cp = ControlPanel(title='Choose Plot Plugins',children=[plugin1,plugin2,plugin3])
        link((self.more,"value"),(cp,"visible"))
        return cp

    def user_function_panel(self):
        f = Text(description="Function, Args:")
        link((self.model,"user_f"),(f,"value"))
        fapp = Button(description = "Apply")
        fapp.on_click(lambda x: self.model.apply_userf(name='Apply clicked'))
        cp2 = ControlPanel(title='User Defined Function',children=[f,fapp])
        link((self.more,"value"),(cp2,"visible"))
        return cp2



### Enthought tool suite imports 
from traits.api import Instance, Str, Enum, Range, HasTraits, Button, Enum, Property, Bool, Any, on_trait_change
from traitsui.api import Item, View, HGroup, VGroup, Group, Include, ShellEditor
from enable.api import ComponentEditor
from chaco.api import LabelAxis, Plot, ToolbarPlot, PlotAxis, LinearMapper, ArrayDataSource, DataRange1D
from chaco.tools.api import BetterSelectingZoom, PanTool
from random import randint
from copy import deepcopy
from pandas import DataFrame

from chaco.tools.api import RangeSelection, RangeSelectionOverlay

### For testing 
import numpy as np
from time import sleep

### For skspec imports
#from skspec import get_csvdf
#from skspec import wavelength_slices
#from skspec import datetime_convert
from pandas.stats.moments import rolling_mean

### Local import 
from pandasplotdatav3 import PandasPlotData


class DefaultEnum(Enum):
    ''' Enum trait with default value.'''
    def __init__(self, *args, **kwds):
        super(DefaultEnum, self).__init__(*args, **kwds)
        if 'default' in kwds:
            self.default_value = kwds['default']


### Add a method to relable rows/columns in place?
class PandasPlot(HasTraits):
    '''Plot to interface a pandas df'''


    ### For testing
    rnd_cols=Bool(False)
    samecols=Bool(False)
    transpose=Bool(False)

    ### Traits of the actual plot and plotdatasource handler ###
    _dffull=Instance(DataFrame)  #Any?    
    plot=Instance(Plot)  
    plotdata=Instance(PandasPlotData) #Data stored at any given time in the plot
    df=Instance(DataFrame)  


    ### Axis traits    
    idxname=Str('Index')
    colname=Str('Column')
    idxorient=DefaultEnum('top', 'bottom', 'left', 'right', default='bottom')
    colorient=DefaultEnum('top', 'bottom', 'left', 'right', default='top')
    selection_axis=DefaultEnum('index','value', default='index') #Which axis does selection tool sample

    ### Aesthetic traits
    title=Str('Title') #Defaults from df name
    linecolor=Str('yellow')
    pointcolor=Str('red')
    linestyle=Enum('line', 'scatter', 'both')
    markersize=Range(low=1, high=5)
    
    sampling=Range(low=0.0, high=100., value=100.0)
    _spacing=Property(depends_on='sampling')



    ### _Event Handlers
    
    def _rnd_cols_changed(self):

        x=np.linspace(0, 100, 100) #Index generator
        y=np.linspace(0, 100, 100) #Column generator

        scale=randint(1,1000)        

        self.df=DataFrame((np.random.rand(len(x),len(y))), columns=(scale/2.0)*y, index=scale*x)


    def _samecols_changed(self):    
        self.df=DataFrame((np.random.rand(100,100)))

    def _transpose_changed(self):
        self.df=self.df.transpose()

    ### Axis Aesthetics
    @on_trait_change('idxname','idxcolumn', 'idxorient', 'colorient')
    def _axis_changed(self, trait, new):
        '''Change plot axis name or orientation'''
        setattr(self, trait, new)
        self._update_axis()
        self.plot.request_redraw() #Necessary but how do I only call axis redraw?
        
    def _title_changed(self, old, new):
        if old != new:
            self.plot.title=new
            self.plot.request_redraw() #HOW TO PREVENT COLLISION WITH TOP OVERLAY
        
    def _selection_axis_changed(self, old, new):
        if old != new:
            self._update_lines()



    def _df_changed(self, old, new):
        ''' Handles how updates occur when df changes.  Evaluates if columns or columnlabels
        have been changed.  Provides entry condition as well.

        Note: New automatically sets self.df, so when I refer to new, I am actually
        referring to self.df.  Using "new" instead of self.df is just for readability'''

        ### Initialize plot first time df is passed into the class.  Boolean listeners
        ### for df behave oddly, so uses self.plotdata for entry condition.

        if not self.plotdata:
            self.plotdata=PandasPlotData(df=new)

            self._dffull=new

            ### Try to infer plot title from df name ###	    
            try: 
                self.title=new.name   #Turn into a "transfer" method where I pass list of attributes i want look for
            except AttributeError:
                pass

            ### Draw barebones of plot
            self._plot_default()
            self._update_lines()           


        ### Decide to update columns or completely redraw df.  
        else:
            changed=self.plotdata.set_df(new)

            ### If 'event changed', don't bother updaing lines
            if not changed:
                self._update_lines()
    
    ### Plot spacing
    
    def _get__spacing(self):
        '''Integer spacing given % sampling'''
        samp=self.sampling/100.0
        colsize=df.shape[0]
        return colsize - int(round( colsize*samp, 0))
    
    def _sampling_changed(self, old, new):
        if old != new:                 
            self._update_samples()
                
    def _update_samples(self):
        '''Updates the list of line plots shown or hidden based on plot sampling.'''
        if self._spacing==0:
            return           

        ### Hide all plots
        to_hide=self.plot.plots.keys()
        self.plot.hideplot(*to_hide)   
        
        ### Show only plots in samples        
        to_show=self.plot.plots.keys()[::self._spacing]
        self.plot.showplot(*to_show)
        self.plot.request_redraw()
                
    def _update_rangeselect(self):
        ''' Overwrites range selection tool in current plot.'''
           
        #### Remove current overlay
        self.plot.overlays=[obj for obj in self.plot.overlays if not isinstance(obj, RangeSelectionOverlay)]
            
        mycomp=self.plot.plots.itervalues().next()[0] #Quick wayt to get first value in dictionary
        
        inds=range(len(self.df.index))
        idx=ArrayDataSource(inds)
        vals=ArrayDataSource(df.index.values)
        
        index_range = DataRange1D(idx)        
        val_range=DataRange1D(vals)
        imap=LinearMapper(range=index_range)#,stretch_data=self.index_mapper.stretch_data)     
        vmap=LinearMapper(range=val_range)
     #   mycomp.index_range.refresh()
        
        mycomp.index_mapper=imap        
        mycomp.value_mapper=vmap
        
        self.rangeselect=RangeSelection(mycomp, axis=self.selection_axis)
        self.plot.active_tool = self.rangeselect
        self.plot.overlays.append(RangeSelectionOverlay(component=mycomp)) 
        self.rangeselect.on_trait_change(self.on_selection_changed, "selection")
                

    def _update_lines(self):
        ''' Redraws lines, plots and reapplies line selection.'''


        oldplots=self.plot.plots.keys()
        newplots=[name for name in self.plotdata.list_data(as_strings=True)]

        to_remove=[p for p in oldplots if p not in newplots]
        to_add=[p for p in newplots if p not in oldplots]

        if to_remove:
            for p in to_remove:
                self.plot.delplot(p)
                
        if to_add:
            for name in to_add:
                self.plot.plot(('index', name), name=name, color=self.linecolor)
                                      
        self._update_axis()
        self._update_samples()
        self._update_rangeselect()

    def on_selection_changed(self, selection):
        if selection != None:
            self.rangeXMin, self.rangeXMax = selection    
            print selection


    def _update_axis(self):    
        ''' Forces a label axis onto the plot. '''

        print 'updaing axis', self.idxname

        indexlabels=[str(round(i,1)) for i in self.df.index]
        columnlabels=[str(round(i,1)) for i in self.df.columns]


        index_axis=LabelAxis(self.plot, orientation=self.idxorient, 
                             positions=range(int(float(indexlabels[0])),
                                             int(float(indexlabels[-1]))), 

                             labels=indexlabels,#, resizable='hv',
                             title=self.idxname)



        col_axis=LabelAxis(self.plot, orientation=self.colorient, 
                           positions=range(int(float(columnlabels[0])),
                                           int(float(columnlabels[-1]))), 


                           labels=columnlabels,#, resizable='hv',
                           title=self.colname)        

        ### Remove underlays              
        self.plot.underlays=[obj for obj in self.plot.underlays if not isinstance(obj, PlotAxis)]
        self.plot.underlays=[obj for obj in self.plot.underlays if not isinstance(obj, LabelAxis)]

        self.plot.underlays.append(index_axis)
        self.plot.underlays.append(col_axis)



    def _plot_default(self, toolbar=True, **pltkwds):
        ''' Draw bare plot, including main plotting area, toolbar, etc...
         either at initialization or global redo'''      
        
        if toolbar:
            self.plot=ToolbarPlot(self.plotdata, **pltkwds)
        else:
            self.plot=Plot(self.plotdata, **pltkwds)

        self.plot.title = self.title
        self.plot.padding = 50
        self.plot.legend.visible=False

        self.plot.tools.append(PanTool(self.plot))
        zoom=BetterSelectingZoom(component=self.plot, tool_mode="box", always_on=False)
        self.plot.overlays.append(zoom)
        
    def _overwrite_plotdata(self):
        '''When a new instance of PandasPlotData is created, this overwrites the
        data source and updates the axis values.'''
        self.plotdata=PandasPlotData(df=self.df)
        self._plot_default() #CAN THIS JUST DRAW LINES


    ### Traits View

    main_group=Group(
        HGroup(Item('rnd_cols'), Item('samecols'), Item('transpose'),),
        Item('plot', editor=ComponentEditor(), show_label=False),
        Item('idxname'), Item('idxorient'), Item('selection_axis'), Item('title'),
        Item('sampling'),
        
        #   Item('df_new'), Item('df_change'), 
        #Include('sample_group'),
        #Include('axis_traits_group')
    )

    traits_view=View( Include('main_group') , height=600, width=800)



class WithShell(HasTraits):
    df=Instance(DataFrame)
    shell=Any()
    pandasplot=Instance(PandasPlot)

    def __init__(self):
#        self.df=DataFrame([1,2,3])
        self.pandasplot=PandasPlot()
#        self.pandasplot.df=self.df

    def _df_changed(self):
        if self.pandasplot:
            self.pandasplot.df=self.df


    main_group=Group(
        Item('pandasplot', show_label=False, style='custom'),
        #   Item('df_new'), Item('df_change'), 
        Item('shell', editor=ShellEditor(), style='simple'),
        #Include('sample_group'),
        #Include('axis_traits_group')
    )

    traits_view=View( Include('main_group'), resizable=True, height=600, width=800)

if __name__=='__main__':

    #  df=get_csvdf('spectra.pickle')
    df=DataFrame([1,2,3,4,5], index=[400,500,600,700,760])


    ### THIS WILL INDUCE A FAILURE IN THE PLOT.PLOT call
    #   df=wavelength_slices(df, ranges=((350.0,370.0), (450.0,500.0), (550.0,570.0), (650.0,680.0), (680.0,700.0)),\
    #                           apply_fcn='simps')

    ### resample df to be a bit less dense, makes plot changes more fluid ###

#    df=df.ix[300.0:800.0:5.0, 0:150:5]

#    theplot=PandasPlot()   
#    theplot.df=df   
#    theplot.configure_traits()

#    test=PandasPlot()   

    test=WithShell()
    test.df=DataFrame((np.random.rand(100,100)))

#    for i in range(1000):
#        test.df=df
    #idx=list('abcdefghij')
    #test.df=DataFrame((np.random.rand(10,10)), index=idx)
    test.configure_traits()




### Actually use namedtuple for this, just storing these in case I ever fix pyrecords to take null values.
from collections import namedtuple

metadata_full=( 
             ### File related attributes ###
             ('filecount', int(0) ), #0-256 bins basically
             ('filestart', int(0) ),  
             ('fileend', int(0) ),


	     ### Time related attributes
             ('timestart',''), #datetime(0,0,0)),
             ('timeend',''),

	     ### Wavelength related attributes
             ('specstart', float(0.0)),
             ('specend', float(0.0)),

             ### Spectrometer datafile attributes

             ('spectrometer', ''),
            ('int_unit', ''),
            ('int_time',''),
            ('spec_avg',int(0)),
            ('boxcar',int(0)),
            ('dark_spec_pres',''), #String so can distingush unkown from No or Yes
            ('ref_spec_pres',''),
            ('electric_dark_correct',''),
            ('strobe_lamp',''),
            ('detector_nonlin_correct',''),
            ('stray_light_correct',''),
            ('pix_in_spec', int(0)),  #Int 0 or default to none
            )

MetaData=namedtuple('Spectralmetadata', [i[0] for i in metadata_full])





""" 2d and 3d wrappers for plotting 2d and 3d data in dataframes """
__author__ = "Adam Hughes"
__copyright__ = "Copyright 2012, GWU Physics"
__license__ = "Free BSD"
__maintainer__ = "Adam Hughes"
__email__ = "hugadams@gwmail.gwu.edu"
__status__ = "Development"

import datetime

import matplotlib.pyplot as plt
import matplotlib.ticker as mplticker
import matplotlib.dates as dates
from mpl_toolkits.mplot3d import Axes3D, axes3d, art3d #Need Axes3d for 3d projection!

import numpy as np
import matplotlib.cm as cm
from matplotlib.collections import PolyCollection
from basic_plots import PlotError
from skspec.core.abcspectra import SpecError

import plot_utils as pu
import skspec.config as pvconfig

from skspec.exceptions import badvalue_error

# Smart float to int conversion
_ir=lambda(x): int(round(x))

from skspec.plotting.basic_plots import range_timeplot, areaplot, _genplot    
from skspec.plotting.plot_registry import PlotRegister

_TIMESTAMPPADDING = 2.9 #Padding for timestamp labels
_TIMESTAMPFORMAT = '%H:%M:%S'

# Colormap of wire plot: hack and only works on a square mesh (eg 100x100)
# https://github.com/matplotlib/matplotlib/issues/3562
def wire_cmap(wires, ax, cmap='hsv'):
    """ Add a colormap to a set of wires (returned form ax.plot_wireframe)"""
    # Retrive data from internal storage of plot_wireframe, then delete it

    if wires._segments3d.ndim != 3:
        raise PlotError('Wireframe colormapping for non-squre data (ie same '
                        'number rows and columns) is not supported.')

    nx, ny, _  = np.shape(wires._segments3d)
               
    wire_x = np.array(wires._segments3d)[:, :, 0].ravel()
    wire_y = np.array(wires._segments3d)[:, :, 1].ravel()
    wire_z = np.array(wires._segments3d)[:, :, 2].ravel()
    wires.remove()        
    
    # create data for a LineCollection
    wire_x1 = np.vstack([wire_x, np.roll(wire_x, 1)])
    wire_y1 = np.vstack([wire_y, np.roll(wire_y, 1)])
    wire_z1 = np.vstack([wire_z, np.roll(wire_z, 1)])
    to_delete = np.arange(0, nx*ny, ny)
    wire_x1 = np.delete(wire_x1, to_delete, axis=1)
    wire_y1 = np.delete(wire_y1, to_delete, axis=1)
    wire_z1 = np.delete(wire_z1, to_delete, axis=1)
    scalars = np.delete(wire_z, to_delete)
        
    segs = [list(zip(xl, yl, zl)) for xl, yl, zl in \
                     zip(wire_x1.T, wire_y1.T, wire_z1.T)]
    
    # Plots the wireframe by a  a line3DCollection
    new_wires = art3d.Line3DCollection(segs, cmap=cmap)
    new_wires.set_array(scalars)
    ax.add_collection(new_wires)
    return new_wires

def custom_wireframe(ax, X, Y, Z, *args, **kwargs):
    """
    Overoad matplotlib's plot_wireframe for a special use case that we want
    to plot a wireframe over a surface with customizability of those
    lines.
    In future versions, this may be incorporated into matplotlib natively.
    This would still be required for backwards compatibility.
    """

    rstride = kwargs.pop("rstride", 1)
    cstride = kwargs.pop("cstride", 1)

    ts = Z

    had_data = ax.has_data()
    Z = np.atleast_2d(Z)
    # FIXME: Support masked arrays
    X, Y, Z = np.broadcast_arrays(X, Y, Z)
    rows, cols = Z.shape

    # We want two sets of lines, one running along the "rows" of
    # Z and another set of lines running along the "columns" of Z.
    # This transpose will make it easy to obtain the columns.
    tX, tY, tZ = np.transpose(X), np.transpose(Y), np.transpose(Z)

    if rstride:
        rii = list(xrange(0, rows, rstride))
        # Add the last index only if needed
        if rows > 0 and rii[-1] != (rows - 1) :
            rii += [rows-1]        
    else:
        rii = []

    if cstride:                
        cii = list(xrange(0, cols, cstride))
        if cols > 0 and cii[-1] != (cols - 1) :
            cii += [cols-1]        
    else:
        cii = []

    # If the inputs were empty, then just
    # reset everything.
    if Z.size == 0 :
        rii = []
        cii = []

    xlines = [X[i] for i in rii]
    ylines = [Y[i] for i in rii]
    zlines = [Z[i] for i in rii]

    txlines = [tX[i] for i in cii]
    tylines = [tY[i] for i in cii]
    tzlines = [tZ[i] for i in cii]

    # Row lines from rowstrides
    lines = [list(zip(xl, yl, zl)) for xl, yl, zl in \
             zip(xlines, ylines, zlines)]

    # Col lines form colstrides
    lines += [list(zip(xl, yl, zl)) for xl, yl, zl in \
              zip(txlines, tylines, tzlines)]

    linec = art3d.Line3DCollection(lines, *args, **kwargs)

    ax.add_collection(linec)
    ax.auto_scale_xyz(X, Y, Z, had_data)

    if 'cmap' in kwargs:
        linec = wire_cmap(linec, ax, cmap=kwargs['cmap'])    
    
    return linec


def format_date(x, pos=None):
    return dates.num2date(x).strftime(_TIMESTAMPFORMAT) 


# Rename
def _gen2d3d(*args, **pltkwargs):
    # UPDATE
    """ Abstract layout for 2d plot.
    For convienence, a few special labels, colorbar and background keywords 
    have been implemented.  If these are not adequate, it one can add 
    custom colorbars, linelabels background images etc... easily just by 
    using respecitve calls to plot (plt.colorbar(), plt.imshow(), 
    plt.clabel() ); my implementations are only for convienences and
    some predefined, cool styles.
        
    countours: Number of desired contours from output.
    
    label: Predefined label types.  For now, only integer values 1,2.  Use plt.clabel to add a custom label.
    
    background: Integers 1,2 will add gray or autumn colormap under contour plot.  Use plt.imgshow() to generate
                custom background, or pass a PIL-opened image (note, proper image scaling not yet implemented).

    c_mesh, r_mesh: These control how man column and row iso lines will be projected onto the 3d plot.
                    For example, if c_mesh=10, then 10 isolines will be plotted as columns, despite the actual length of the
                    columns.  Alternatively, one can pass in c_stride directly, which is a column step size rather than
                    an absolute number, and c_mesh will be disregarded.
                    
                
    fill: bool (False)
        Fill between contour lines.

    **pltkwargs: Will be passed directly to plt.contour().

    Returns
    -------
    tuple: (Axes, SurfaceFunction)
        Returns axes object and the surface function (e.g. contours for
        contour plot.  Surface for surface plot.
    
    """
    
    # Use a label mapper to allow for datetimes in any plot x/y axis
    _x_dti = _ = _y_dti = False

    # Passed Spectra
    if len(args) == 1:
        ts = args[0]

        try:
            index = np.array([dates.date2num(x) for x in ts.index])
            _x_dti = True
        except AttributeError:
            index = ts.index.values #VALUES NECESSARY FOR POLY CMAP
            
        try:
            cols = np.array([dates.date2num(x) for x in ts.columns])
            _y_dti = True
        except AttributeError:
            cols = ts.columns.values #VALUES NECESSARY FOR POLY CMAP
                
        yy, xx = np.meshgrid(cols, index)

    # Passed xx, yy, ts/zz
    elif len(args) == 3:
        xx, yy, ts = args
        cols, index = ts.columns.values, ts.index.values
        
    else:
        raise PlotError("Please pass a single spectra, or xx, yy, zz.  Got %s args"
                        % len(args))
             
    # Boilerplate from basic_plots._genplot(); could refactor as decorator
    xlabel = pltkwargs.pop('xlabel', '')
    ylabel = pltkwargs.pop('ylabel', '')
    zlabel = pltkwargs.pop('zlabel', '')    
    title = pltkwargs.pop('title', '')

    labelsize = pltkwargs.pop('labelsize', pvconfig.LABELSIZE) #Can also be ints
    titlesize = pltkwargs.pop('titlesize', pvconfig.TITLESIZE)

    # Choose plot kind
    kind = pltkwargs.pop('kind', 'contour')
    grid = pltkwargs.pop('grid', True)
#    pltkwargs.setdefault('legend', False) #(any purpose in 2d?)
#   LEGEND FOR 2D PLOT: http://stackoverflow.com/questions/10490302/how-do-you-create-a-legend-for-a-contour-plot-in-matplotlib
    pltkwargs.setdefault('linewidth', 1)    
    
    cbar = pltkwargs.pop('cbar', False)
    
    outline = pltkwargs.pop('outline', None)
    if outline:
        if kind != 'surf' and kind != 'waterfall':
            raise PlotError('"outline" is only valid for "surf" and "waterfall"'
                        ' plots.  Please use color/cmap for all other color'
                        ' designations.')
    
    fig = pltkwargs.pop('fig', None)
    ax = pltkwargs.pop('ax', None)  
    fill = pltkwargs.pop('fill', pvconfig.FILL_CONTOUR)  
    
    xlim = pltkwargs.pop('xlim', None)
    ylim = pltkwargs.pop('ylim', None)
    zlim = pltkwargs.pop('zlim', None)
    
    #Private attributes
    _modifyax = pltkwargs.pop('_modifyax', True)
    contours = pltkwargs.pop('contours', pvconfig.NUM_CONTOURS)
    label = pltkwargs.pop('label', None)
    
    projection = None
        
    if kind in PLOTPARSER.plots_3d:
        projection = '3d'

        elev = pltkwargs.pop('elev', 35)
        azim = pltkwargs.pop('azim', -135)        
        
        view = pltkwargs.pop('view', None)
        if view:
            if view == 1:
                elev, azim = 35, -135
            elif view == 2:
                elev, azim = 35, -45 
            elif view == 3:
                elev, azim = 20, -10  # Side view
            elif view == 4:
                elev, azim = 20, -170
            elif view == 5:
                elev, azim = 0,-90          
            elif view == 6:
                elev, azim = 65, -90
            else:
                raise PlotError('View must be between 1 and 6; otherwise set'
                                ' "elev" and "azim" keywords.')

        # Orientation of zlabel (doesn't work...)
        _zlabel_rotation = 0.0
        if azim < 0:
            _zlabel_rotation = 90.0

        if 'mesh' in pltkwargs:
            pltkwargs['c_mesh'] = pltkwargs['r_mesh'] = pltkwargs.pop('mesh')


        # Defaults will be ignored if mesh or ciso in kwargs
        ciso_default = pvconfig.C_MESH
        if len(ts.columns) < ciso_default:
            ciso_default = len(ts.columns)
            
        riso_default = pvconfig.R_MESH  
        if len(ts.index) < riso_default:
            riso_default = len(ts.index)       
        
        
        c_mesh = pltkwargs.pop('c_mesh', ciso_default)
        r_mesh = pltkwargs.pop('r_mesh', riso_default)
        
        if c_mesh > ts.shape[1] or c_mesh < 0:
            raise PlotError('"c_mesh/column mesh" must be between 0 and %s, got "%s"' %
                            (ts.shape[1], c_mesh))

        if r_mesh > ts.shape[0] or r_mesh < 0:
            raise PlotError('"r_mesh/row mesh" must be between 0 and %s, got "%s"' % 
                            (ts.shape[0], r_mesh))
        

        if c_mesh == 0:
            cstride = 0
        else:
            cstride = _ir(ts.shape[1]/float(c_mesh) ) 
            
        if r_mesh == 0:
            rstride = 0
        else:
            rstride = _ir(ts.shape[0]/float(r_mesh) )   
    
                        
        pltkwargs.setdefault('cstride', cstride)
        pltkwargs.setdefault('rstride', rstride)

    elif kind == 'contour':
        pass

    else:
        raise PlotError('_gen2d3d invalid kind: "%s".  '
               'Choose from %s' % (kind, PLOTPARSER.plots_2d_3d))

    # Is this the best logic for 2d/3d fig?
    if not ax:
        f = plt.figure()
#        ax = f.gca(projection=projection)       
        ax = f.add_subplot(111, projection=projection)
        if not fig:
            fig = f
        
    
    
    # PLT.CONTOUR() doesn't take 'color'; rather, takes 'colors' for now
    if 'color' in pltkwargs:       
        if kind == 'contour' or kind == 'contour3d':
            pltkwargs['colors'] = pltkwargs.pop('color')
        
    # Convienence method to pass in string colors
    if 'colormap' in pltkwargs:
        pltkwargs['cmap'] = pltkwargs.pop('colormap')

    if 'cmap' in pltkwargs:
        if isinstance(pltkwargs['cmap'], basestring):
            pltkwargs['cmap'] = pu.cmget(pltkwargs['cmap'])
    
    # Contour Plots    
    # -------------

        # Broken background image
        ### More here http://matplotlib.org/examples/pylab_examples/image_demo3.html ###
        # Refactored with xx, yy instead of df.columns/index UNTESTED
        #if background:
            #xmin, xmax, ymin, ymax = xx.min(), xx.max(), yy.min(), yy.max()
            
            ## Could try rescaling contour rather than image:
            ##http://stackoverflow.com/questions/10850882/pyqt-matplotlib-plot-contour-data-on-top-of-picture-scaling-issue
            #if background==1:
                #im = ax.imshow(ts, interpolation='bilinear', origin='lower',
                            #cmap=cm.gray, extent=(xmin, xmax, ymin, ymax))             
    
        #### This will take a custom image opened in PIL or it will take plt.imshow() returned from somewhere else
            #else:
                #try:
                    #im = ax.imshow(background) 
                #### Perhaps image was not correctly opened    
                #except Exception:
                    #raise badvalue_error(background, 'integer 1,2 or a PIL-opened image')


    # Note this overwrites the 'contours' variable from an int to array
    if kind == 'contour' or kind == 'contour3d':

        # Cornercase datetimeindex and offest from add projection hack
        try:
            pltkwargs['offset'] = dates.date2num(pltkwargs['offset'])
        except Exception:
            pass

        if fill:                           #Values of DTI doesn't work
            mappable = ax.contourf(xx, yy, ts.values, contours, **pltkwargs)    #linewidths is a pltkwargs arg
        else:
            mappable = ax.contour(xx, yy, ts.values, contours, **pltkwargs)    


        ### Pick a few label styles to choose from.
        if label:
            if label==1:
                ax.clabel(inline=1, fontsize=10)
            elif label==2:
                ax.clabel(levels[1::2], inline=1, fontsize=10)   #label every second line      
            else:
                raise PlotError(label, 'integer of value 1 or 2')

 
    elif kind == 'surf': 
        mappable = ax.plot_surface(xx, yy, ts, **pltkwargs)

        if outline:
            try:
                pltkwargs['cmap'] = pu.cmget(outline)
            except Exception: #Don't change; attribute error fails when outline=None
                pltkwargs['color'] = outline     
                pltkwargs.pop('cmap') 

        custom_wireframe(ax, xx, yy, ts, **pltkwargs)
        # Wires are thrown out, since mappable is the surface, and only mappable returned

    elif kind == 'wire':
        pltkwargs.setdefault('color', 'black')        
        mappable = custom_wireframe(ax, xx, yy, ts, **pltkwargs)

    elif kind == 'waterfall':
        
        # Parse outline color (if colormap, error!)
        try:
            pu.cmget(outline) 
        except Exception:
            pltkwargs['edgecolors'] = outline
        else:
            raise PlotError('Waterfall "outline" must be a solid color, not colormap.')
        
        pltkwargs.setdefault('closed', False)
        alpha = pltkwargs.setdefault('alpha', None)

        # Need to handle cmap/colors a bit differently for PolyCollection API
        if 'color' in pltkwargs:
            pltkwargs['facecolors']=pltkwargs.pop('color')
        cmap = pltkwargs.setdefault('cmap', None)
        
        if alpha is None: #as opposed to 0
            alpha = 0.6 * (13.0/ts.shape[1])
            if alpha > 0.6:
                alpha = 0.6        
        
        #Delete stride keywords (waterfall doesn't have strides: not a surface!)
        for key in ['cstride', 'rstride']:
            try:
                del pltkwargs[key]
            except KeyError:
                pass
        
        # Verts are index dotted with data
        verts = []
        for col in ts.columns:  
            values = ts[col]
            values[0], values[-1] = values.min().min(), values.min().min()
            verts.append(list(zip(ts.index, values)))
    
        mappable = PolyCollection(verts, **pltkwargs)
        
        if cmap:
            mappable.set_array(cols) #If set array in __init__, autogens a cmap!
            mappable.set_cmap(pltkwargs['cmap'])

        mappable.set_alpha(alpha)      
                    
        #zdir is the direction used to plot; dont' fully understand
        ax.add_collection3d(mappable, zs=cols, zdir='x' )      

        # custom limits/labels polygon plot (reverse x,y)
        if not ylim:
            ylim = (max(index), min(index))  #REVERSE AXIS FOR VIEWING PURPOSES

        if not xlim:
            xlim = (min(cols), max(cols))    #x 

        if not zlim:
            zlim = (min(ts.min()), max(ts.max()))  #How to get absolute min/max of ts values
            
        # Reverse labels/DTI call for correct orientaion HACK HACK HACK
        xlabel, ylabel = ylabel, xlabel    
        _x_dti, _y_dti = _y_dti, _x_dti
        azim = -1 * azim

    # General Features
    # ----------------
    
    # Some applications (like add_projection) shouldn't alther axes features
    if not _modifyax:
        return (ax, mappable)

    if cbar:
        # Do I want colorbar outside of fig?  Wouldn't be better on axes?
        try:
            cb = fig.colorbar(mappable, ax=ax)
            # Label colorbar on contour since no 3d-zlabel
            if kind == 'contour':
                cb.set_label(zlabel)
        except Exception:
            raise PlotError("Colorbar failed; did you pass a colormap?")
               
    if grid:
        if grid == True:
            ax.grid()
        else:
            ax.grid(color=grid) #Let's any supported color in     
          

    # Format datetime axis
    # -------------------
    if _x_dti:
        ax.xaxis.set_major_formatter(mplticker.FuncFormatter(format_date))
            
        # Uncomment for custom 3d timestamp orientation
 #       if projection:
 #           for t1 in ax.yaxis.get_ticklabels():
 #               t1.set_ha('right')
 #               t1.set_rotation(30)        
 #           ax.yaxis._axinfo['label']['space_factor'] = _TIMESTAMPPADDING

    if _y_dti:
        ax.yaxis.set_major_formatter(mplticker.FuncFormatter(format_date))

        # Uncomment for custom 3d timestamp orientation
  #      if projection:
  #          for t1 in ax.yaxis.get_ticklabels():
  #              t1.set_ha('right')
  #              t1.set_rotation(30)        
  #          ax.yaxis._axinfo['label']['space_factor'] = _TIMESTAMPPADDING

    if xlim:
        ax.set_xlim3d(xlim)

    if ylim:
        ax.set_ylim3d(ylim)        
     
    if zlim:
        ax.set_zlim3d(zlim)    
                
    # Set elevation/azimuth for 3d plots
    if projection:        
        ax.view_init(elev, azim)                 
        ax.set_zlabel(zlabel, fontsize=labelsize, rotation= _zlabel_rotation)  
    
    ax.set_xlabel(xlabel, fontsize=labelsize)
    ax.set_ylabel(ylabel, fontsize=labelsize)
    ax.set_title(title, fontsize=titlesize) 
        
    # Return Ax, contours/surface/polygons etc...
    return (ax, mappable)  


# Support images or any other 2d plot besides contours?
def add_projection(ts, plane='xz', flip=False, fill=False, **contourkwds):
    """ Add a contour plot/projection onto the xy, xz, or yz plane of a 3d
    plot.
    
    args: 2D Arrays or Contour Set
        xx, yy, zz : 2D mesh arrays, or contour set.
    
        
    plane: str ('xz')
        'xz', 'yz', 'xy', choose the plane on which to project the contours.
        
    flip : bool (False)
        The 3d plot is a square of 6 sides.  Flip will change the projeciton
        from the left wall to right wall, or from floor to ceiling in the 
        same plane.
        
    fill : bool (False)
        Fill the contours

    **contourkwds
        Keywords passed directly to ax.contour.  

    Returns
    -------
    tuple: Axes
        Returns axes object.
    """
                
    plane = plane.lower()

    if plane == 'zx': plane = 'xz'
    if plane == 'zy': plane = 'yz'
    if plane == 'yx': plane = 'xy'
    
    if plane == 'xy':
        offset=ts.min().min()
        if flip:
            offset=ts.max().max()
        zdir = 'z'
      
    elif plane == 'yz':
        offset=ts.index.min()
        if flip:
            offset=ts.index.max()
        zdir = 'x'        

    elif plane == 'xz':
        offset = ts.columns.max()
        if flip:
            offset = ts.columns.min()
        zdir = 'y'           
    
    else:
        raise PlotError('Invalid plane "%s": must be "xy, xz or yz".' % plane)
    
          
    ax, cset = _gen2d3d(ts, 
                        zdir=zdir, 
                        kind='contour', 
                        offset=offset, 
                        fill=fill,
                        _modifyax=False,
                        **contourkwds)  
   
    # Return axes only as this is public
    return ax 
   
   
def spec3d(ts, projection=True, fill=True, samples=5, contourkwds={}, **pltkwargs):
    """ Wireframe plot with no connected clines.  By default, adds an xz 
    projection. 
    
    Parameters
    ----------
    projection : color/colormap
        Valid colormap or solid color of contour projection.
        
    samples : int
        Number of samples to take from dataset.  Defaults to 5.  Must
        be within 0 and the number of columns in Spectra.
        
    contourkwds: {}
        Dictionary that holds arguments specific to the projection.
    
    Returns
    -------
    tuple: (Axes, SurfaceFunction)
        Returns axes object and the surface function (e.g. contours for
        contour plot.  Surface for surface plot.)
    
    Notes
    -----
    Mostly a shortcut for a very useful 3d look at data. Thus, did not add
    much customizability as this plot can be assembled from _gen2d3d2d and
    add_projeciton.
    """


    for invalid in ['c_mesh', 'r_mesh', 'cstride', 'rstride']:
        if invalid in pltkwargs:
            raise PlotError('Unsupported Keyword %s.'
                            'Please use the samples argument' % invalid)
    #3d Plot
    pltkwargs['kind'] = 'wire'
    pltkwargs['r_mesh'] = 0
    pltkwargs['c_mesh'] = samples
    ax, mappable = _gen2d3d(ts, **pltkwargs)
    
    # Projection
    if 'alpha' not in contourkwds:
        contourkwds['alpha'] = 0.3
        
    if projection:
        if projection == True:
            projection = pvconfig.PROJECTION_CMAP

        ax = add_projection(ts, ax=ax, fill=fill, **contourkwds)
   
    return ax, mappable
    
   
# SET REGISTER HERE!  HAS TO BE HERE BECAUSE GEN2D USES IT, SO GET CIRCULAR IMPORT ISSUES
# IF PUT THIS IN A SEPARATE MODULE.  STRICTLY SPEAKING, GEND2D IS MANY PLOTS SO IT HAS
# TO INSPECT ITS OWN KIND ARGUMENT.  THIS IS ONE HEADACHE OF SUCH A DESIGN PATTERN!
PLOTPARSER = PlotRegister()

#Basic plots
PLOTPARSER.add('spec', _genplot, False, 'Spec vs. Variation' )
PLOTPARSER.add('area', areaplot, False, 'Area vs. Variation' )
PLOTPARSER.add('range_timeplot', range_timeplot, False, 'Slice Ranges vs. Variation' )

#Advanced plots
PLOTPARSER.add('contour', _gen2d3d, False, 'Contour Plot' )
PLOTPARSER.add('contour3d',_gen2d3d, True, '3D Contour Plot')
PLOTPARSER.add('wire', _gen2d3d, True, '3D Wireframe')
PLOTPARSER.add('surf', _gen2d3d, True, '3D Surface' )
PLOTPARSER.add('waterfall', _gen2d3d, True, '3D Waterfall' )
PLOTPARSER.add('spec3d', spec3d, True, '3D Wire + Projection' )

if __name__ == '__main__':

    from matplotlib import rc
    from skspec.data import aunps_glass, aunps_water, solvent_evap
    
    ts = aunps_glass()
#    ts = ts.nearby[400:700]
#    ts = ts.nearby[1520:1320]
#    ts=ts.iloc[450:500, 50:100]
#    xx,yy = ts.meshgrid()
    
    
    print PLOTPARSER
    
    ##---- First subplot
    #ax2 = fig.add_subplot(1, 2, 1, projection='3d')
    #_gencorr2d(xx, yy, ts, 
               #fill=True,
               #title='My baller plot',
               #xlabel=ts.full_varunit,
               #ylabel=ts.full_specunit,
               #contours=20,
               #cbar = True,
               #background=False)
               # Is this the best logic for 2d/3d fig?
               
     
    ts = ts.iloc[0:100, :]#.as_varunit('m')

    ts.plot(kind='spec3d', fill=False)    
        
        
        
    ax = ts.plot(
                kind='spec3d',
                cmap = 'hot',
                outline='cool',
    
#                outline='je',
#                outline = 'r',
                
#                cmap='jet',
                cbar=False,
#                c_mesh=5,
#                r_mesh=5,
#                edgecolors='r',
    
#edgecolors='jet',
                linewidth=2,
                alpha=.5,
                contours=5,
                xlabel = ts.full_specunit,
                ylabel = ts.full_varunit,
                zlabel = ts.full_iunit) 
    
    ## Done automatically by spec.plot
#    if ts.index[0] > ts.index[-1]:
#       ax2.set_xlim(ax2.get_xlim()[::-1]) 
    


#    add_projection(ts, ax=ax2, cmap='cool')

    rc('text', usetex=True)    
    
    print ts.shape
    plt.show()



from copy import deepcopy, copy
import numpy as np
from pandas import DatetimeIndex, Timestamp

from skspec.core.abcindex import ConversionIndex, _parse_conversion_unit
from skspec.units.abcunits import UnitError
from skspec.units.intvlunit import INTVLUNITS, TimeDelta, DateTime, \
    DatetimeCanonicalError

class TimeIndex(ConversionIndex):
    """ Stores time labels as Timestamps, Time Deltas or cumulative intervals
    ie seconds, minutes, days from t=0.  Timestamps (e.g. datetimes) are 
    stored as objects, while seconds, minutes etc.. are stored as floats.  
    Therefore, some extra logic is in place to fix breaking points with the
    dtype would otherwise cause errors, mostly the case when dataframe does
    slicing or indexing.
    """
    unitdict = INTVLUNITS
    cumsum = True

    # Overload because datetime and interval need different array types
    # i.e. seconds --> float64 while dti ---> timestamp
    def __new__(cls, input_array, unit=None):
        """ Unit is valid key of unitdict """

        # Is datetimeindex stored on object, or is object itself datetimeindex
        datetimeindex = None 
        #NEEDS TO BE _STORED DTI FOR __new__, DATETIMEINDEX IS A PROPERTY!
        try:
            datetimeindex = input_array._stored_dti 
        except AttributeError:
            pass

    
        # IF INPUTARRAY COMES IN AS A PANDAS DATETIMEINDEX, FORMATS IT
        # TO AN ARRAY OF DATETIMES THEN TO AN ARRAY OF TIMESTAMPS,
        # THE ARRAY OF TIMESTAMPS IS NECESSARY FOR TIMEINDEX
        if isinstance(input_array, DatetimeIndex):
            #Convert datetimes to timestamp
            if unit != 'dti' and unit != None:
                raise IndexError("When creating TimeIndex from DatetimeIndex"
                     " unit must be 'dti' or None, recived %s" % unit)
            datetimeindex = input_array            
            input_array = np.array(input_array.to_pydatetime())
            input_array = np.array([Timestamp(x) for x in input_array])

            # Could force unit = DTI at this point, but are there cases
            # where they want to retain unit = None? 
            # Decided default behavior should be DTI and None set if desired
            unit = 'dti'

                        
        if unit:
            # dti, timedelta
            if unit == 'intvl' or unit == 'dti':
                dtype = 'object'
            else:
                dtype = 'float64'
                # FLOAT ROUNDING
#                input_array = np.around(input_array, decimals=6)

            obj = np.asarray(input_array, dtype=dtype).view(cls)   
        else:
            obj = np.asarray(input_array).view(cls)   

        if datetimeindex is not None:
            obj._stored_dti = datetimeindex

        # Ensure valid unit
        obj._unit = _parse_conversion_unit(unit, cls.unitdict)
        if cls.cumsum:
            obj.cumsum = True
        return obj   

    # Remove
    def _validate_slicer(self, *args, **kwargs):
        """ Trips up by operations like loc/iloc."""
        pass


    def convert(self, outunit):
        """Converts spectral units (see abcindex.convert()).  Handles special
        case of converting to datetimeindex, since the DateTime unit does
        not have conversions; instead the datetimeindex is stored in this
        TimeIndex class and needs to be handled separately.
        """

        inunit = self._unit.short        
        
        # Corner case: None to DTI --> DTI constructor accepts most anything so
        # Enforce that user has set datetimeinex manually
        if outunit == 'dti' and inunit is None:
            try:
                self.datetimeindex
            except AttributeError:
                raise IndexError("Going from None to DTI can result in incorrect datetimeindex."
                        " therefore, please construct index from datetime index from begining.")
        # Hack: when going from None to seconds or another float unit, 
        # the array dtype will change.  This isn't an issue for ABCIndex
        # which is always float type.

        # Handle non-dti conversion and conversions involvin DTI and None

        #if inunit is None and outunit is not None and outunit != 'dti' and self.dtype !='float':
            #raise IndexError("Converting TimeIndex from None to float type"
                  #"is not allowed; please set the timeindex unit first.")

        try:
            out = super(TimeIndex, self).convert(outunit)
        except DatetimeCanonicalError:
 
            # DTI (just pass self.datetimeindex into the constructor)
            if outunit == 'dti':
                if self.datetimeindex is None:
                    raise IndexError("Datetime index not stored; cannot convert to dti")
                return self.__class__(self.datetimeindex, unit = 'dti')
    
            #DTI TO SOMETHING ELSE            
            elif outunit != 'dti' and inunit == 'dti':
                nanoseconds = np.diff(self.datetimeindex.asi8)  #asi8 only defined on DatetimeIndex      
                seconds = nanoseconds * 10**-9  
                seconds = np.insert(seconds, 0, seconds[0]-seconds[0])
                if self.cumsum:
                    seconds = seconds.cumsum()
                # Seconds to datetime to new unit (hacky/three steps)
                out = self.__class__(seconds, unit=outunit)  #THIS MAKES NP ARRAY SO ADD DATETIMEINDEX
                out = self.unitdict[outunit].from_canonical(out)           
                out = self.__class__(out, unit=outunit)

            # Should never happen
            else:
                raise IndexError("SOME LOGIC APPARENTLY NOT ACCOUNTED FOR")
        
        # all conversions go through numpy arrays, so need to manually assign DTI!
        #NEEDS TO BE _STORED DTI FOR __new__, DATETIMEINDEX IS A PROPERTY!        
        try:
            out._stored_dti = self.datetimeindex
        except AttributeError:
            pass
        
        return out


    @property
    def datetimeindex(self):
        try:
            return self._stored_dti
        except AttributeError:
            raise AttributeError("DatetimeIndex not stored.")


    @datetimeindex.deleter
    def datetimeindex(self):
        """ Delete stored datetimeindex.  Not sure if useful, but code will
        breatk if I allow this set to None, so must delete."""
        del(self._stored_dti)
            
        
    @datetimeindex.setter
    def datetimeindex(self, dti):
        
        if not isinstance(dti, DatetimeIndex):
            try:
                dti = DatetimeIndex(dti)
            except Exception:
                raise IndexError('Could not store DatetimeIndex; wrong type %s' \
                                 % type(dti))
            else:
                if len(self) != len(dti):
                    raise IndexError("Length mismatch between passed"
                         "datetimeindex %s and object %s" % (len(dti), len(self)))
                
        self._stored_dti = dti


    def __getslice__(self, start, stop) :
        """This solves a subtle bug, where __getitem__ is not called, and all
        the dimensional checking not done, when a slice of only the first
        dimension is taken, e.g. a[1:3]. From the Python docs:
           Deprecated since version 2.0: Support slice objects as parameters
           to the __getitem__() method. (However, built-in types in CPython
           currently still implement __getslice__(). Therefore, you have to
           override it in derived classes when implementing slicing.)
        """
        return self.__getitem__(slice(start, stop))

    def __getitem__(self, key):
        """ When slicing, the datetimeindex should also be sliced. Otherwise,
        when converting to datetimeindex, could have length mismatch.  EG:
        
        min = ts.convert('m')[0:5]
        len(min) = 5
        dti = min.convert('dti')
        len(dti) = <length of original array>
        """
        out = super(TimeIndex, self).__getitem__(key)
        # If not returning a single value (eg index[0])
        if hasattr(out, '__iter__'):
            # Slice datetime index as well
            #NEEDS TO BE _STORED DTI FOR __new__, DATETIMEINDEX IS A PROPERTY!            
            try:
                out._stored_dti = self.datetimeindex.__getitem__(key) 
            except AttributeError:
                pass
        return out
    
    def copy(self, *args, **kwargs):
        """ Datetimeindex not properly stored when running various copy routines
        either from dataframe.copy or import copy, so manually overriding.
        
        Args kwargs passed to copy (e.g. deep =True)
        """
        outdict = super(TimeIndex, self).copy( *args, **kwargs)
        try:
            outdict.__dict__['_stored_dti']= self.datetimeindex
        except AttributeError:
            pass
        return outdict


    @property
    def is_all_dates(self):
        """ Overwirte this Index method because it's source of many issues
        deep in cython level.  Basically, only should be called if python
        objects.  Otherwise, get ValueError:
             Buffer dtype mismatch, expected 'Python object' but got 'double'
        """
        # Don't ever want this True, or Series will convert ot DateTimeIndex
        return False

        # How I used to do it, but resulted in bug #146
        # https://github.com/hugadams/skspec/issues/146
        #if self.dtype != 'object':
            #return False
        #else:
            #return super(ConversionIndex, self).is_all_dates    


    # Hack to return _engine type correctly for mixed index objects like TimeIndex
    @property
    def _engine_type(self):
        import pandas.index as _index      
        if self.dtype == 'object':
            return _index.ObjectEngine
        elif self.dtype == 'float64':
            return _index.Float64Engine
        else:
            raise IndexError("Not sure which Engine to return for dtype %s"\
                             % self.dtype)      

#    def _validate_slicer(self, key, f):
#        pass


if __name__ == '__main__':
    from pandas import date_range

    idx = TimeIndex([1,2,3])
    idx = idx.convert('s')
    idx = idx.convert('m')

    #From datetime constructor
    idx = TimeIndex.from_datetime(date_range(start='3/3/12',periods=30,freq='45s'))
    print idx
    for unit in INTVLUNITS:
        print unit, idx.convert(unit)

    print idx.unitdict.keys()

#   idx.cumsum = False
#   for unit in INTVLUNITS:
#      print unit, idx.convert(unit)   


''' Interface to make a panel-type object for storing timespectra.  Allows for
    axis aliases like "spectral" and "temporal" slicers.'''

from pandas.core.panel import Panel
from skspec.core.timespectra import TimeSpectra

class SpecPanel(Panel):
    
    _constructor_sliced = TimeSpectra
    
    def __init__(self, data=None, items=None, spectral=None, temporal=None,
                        copy=False, dtype=None, name=None):
    
        '''Initialize with same arguments as panel, with special keywords
          for the SpecStack object.
          
         Parameters:
        -----------
           See Panel API for Panel__init__() arguments.
           name: name of the SpecStack object.'''    
          
        super(SpecPanel, self).__init__(data=data, items=items, major_axis=
                           spectral, minor_axis=temporal, copy=copy,
                           dtype=dtype)      
       

SpecPanel._setup_axes(axes      = ['items', 'spectral', 'temporal'], 
                  info_axis = 0,
                  stat_axis = 1,
                  aliases   = { 0: 'index',
                                1: 'columns' },
                  slicers   = { 'spectral': 'index',
                                'temporal': 'columns' })


from skspec.plotting.plot_utils import splot

# Advanced plots has to create plotparser because its plot methods depend on it
# Surely just bad design
from skspec.plotting.advanced_plots import _gen2d3d, spec3d, PLOTPARSER, \
         add_projection

from skspec.plotting.basic_plots import range_timeplot, areaplot, _genplot    

from skspec.plotting.multiplots import quad_plot, six_plot




import os.path as op

__docformat__ = 'restructuredtext' #What's this actually do

### Core classes
from skspec.core.spectra import Spectra
from skspec.core.timespectra import TimeSpectra
from skspec.core.anyspectra import AnyFrame
from skspec.core.specstack import SpecStack

from skspec.units.abcunits import Unit

#  Borrowing API from skimage
pkg_dir = op.abspath(op.dirname(__file__))
data_dir = op.join(pkg_dir, 'data')
bundled_dir = op.join(pkg_dir, 'bundled')

#Abspath will merge the .. and go up and into examples
examples_dir = op.abspath(op.join(pkg_dir, '../examples/Notebooks'))



import numpy as np
from chaco.api import AbstractPlotData
from traits.api import Dict, Instance, Str, Array, Enum
from collections import OrderedDict #Req python 2.7
from operator import itemgetter

### Imports for testing ###
from pandas import DataFrame

### ADD A MODIFYING FUNCTION THAT DISTINGUISHES BETWEEN DATAFRAME AND SERIES COLUMN GETTERS ###
class PandasPlotData(AbstractPlotData):

    ''' Chaco requires a PlotData interface to manage plot/data mapping; however, pandas
    already is its own nice data handler, so this class mostly provides a wrapper that tries
    to be a hidden middle man.  As such, I try to make this PandasPlotData feel like a pandas
    dataframe.  It would be nice to have chaco communicate with a dataframe directly, but for now
    it must do so through this liason class.'''

    # Dict to store extra dataframes or series that aren't necessarily in the dataframe
    # Must pass
    arrays=OrderedDict()  #Str(name): name, index, data
    
    #Define a dictionary called extras which stores array data that is not belonging to the dataframe 
    #But compliments it.  Helps separate dataframe data from additional data used in plot.  
    #Useful to put here, instead of its own arrayplotdata class because then a single plot can
    #access both types of data
    extras=OrderedDict()

    #Itemgetters for easier access to nested data in the arrays/extras dictionaries
    nameget=itemgetter(0)
    idxget=itemgetter(1)
    dataget=itemgetter(2)       
    
    #Stores labels in their own, special arrays, unmodified.  More useful in subclasses.
    primaryaxis=Enum(0, 1)
    primaryname=Str('columns')
    secondaryname=Str('index')
    _primarylabel=Array()   #Don't forget to convert list--array to remove Index object functioality
    _secondarylabel=Array()

    # Dict mapping data series name to a mask array holding the selections. (Untested)
    selections = Dict()

    def __init__(self, dataframe, extras=None, add_labels_to_extras=False, **kwtraits):
        """PandasPlotData exposes a PlotData interface from a DataFrame.
        It is chosen that this object MUST be initialized with a dataframe.

        This class is intentionally constructed to imitate a dataframe.  Therefore,
        it is built to accept new dataframes instead of adding or removing columns.
        If a dataframe is modified in place such that its primary axis label is unchanged,
        then changes are overwritten with set_data.
        
        An additional trait called "extras" behaves more like an arrayplotdata container, meaning
        data can be added at will.  Therefore, persistence between the actual dataframe and the
        psuedo dataframe (arrays) maintained by this class is emulated.
        
        This class allows users to call get_data using non-sring names, but ensures that plotting
        programs can call by strings without any problem.  
        
        Once the primary data axis is passed in instantiation, the orientation if fixed and can't be 
        flipped at another time.
        
        If **add_labels_to_extras, this will automatically store row/column labels as data arrays.
        This is useful if these axis contain numerical data that needs to be plotted.
        """

        super(PandasPlotData, self).__init__(**kwtraits) #Trait initialization 
        if len(dataframe.shape) > 2:
            raise NotImplementedError('Multidimensional dataframes of order 3 or higher \
	    are not supported by in PandasPlotData')
        
        if extras:
            if len(extras.shape) > 2:
                raise NotImplementedError('Multidimensional dataframes of order 3 or higher \
                are not supported by in PandasPlotData')            
        
        if self.primaryaxis == 0:
            pass
        elif self.primaryaxis == 1:
            dataframe=dataframe.transpose()
            if extras:
                extras.transpose()
        else:
            raise(AttributeError('Attribute "primaryaxis" must be 0, 1 but %s was passed'%axis))
              
        self.arrays=self._set_arrays(dataframe)
        event={'added':self.arrays.keys()}  
        if extras:
            self.extras=self._set_arrays(extras)
            event['added'].extend(self.extras.keys())
            
        self.data_changed = event        

        ### Store label array separately and write to extras if desired
        self._primarylabel=np.asarray(list(dataframe.columns))  
        self._secondarylabel=np.asarray(list(dataframe.index))
        if add_labels_to_extras:
            self.set_extra(self.primaryname, self._primarylabel)
            self.set_extra(self.secondaryname, self._secondarylabel)


    #------------------------------------------------------------------------
    # Dictionary Interface that lets me treat the PandasPlotData object like
    # a dictionary.  Aka can do print pandasplotdata['array1'] instead of 
    # pandasplotdata.get_data('array1')
    #------------------------------------------------------------------------

    def __getitem__(self, name):
        return self.get_data(name)

    #Not sure I want to use the set/del in the dictionary interface until I 
    #know the best way to distinguish between extras and arrays in these respects.
    def __setitem__(self, name, value):
        raise NotImplemented
        #return self.set_extra(name, value)

    def __delitem__(self, name):
        raise NotImplemented        
        #return self.del_data(name)
    

    def list_data(self, as_strings=False):
        """ Returns a list of the names of the selections managed by this instance.  For convienence,
        axis can be 0,1 or the index label column label.
        """
        if as_strings:
            return self.arrays.keys()
        else:
            return [self.nameget(val) for val in self.arrays.values()]


    def get_data(self, name):
        """ Returns the array associated with *name*.  First checks self.arrays,
        then checks self.extras.  """
        try:
            value=self.arrays[str(name)]
        except KeyError:
            try:
                value=self.extras[str(name)]            
            except KeyError:
                return None
                
        return self.dataget(value) #Return array data

    def del_data(self, name):
        """ Deletes the array specified by *name*, or raises a KeyError if
        the named array does not exist.
        """
        raise NotImplementedError('Do I want to delete data from a dataframe?')
        #try:
            #del self.arrays[str(name)]
        #except KeyError:
            #raise KeyError("Data series '%s' does not exist." % name)
                                

    def set_data(self, name, new_data):
        """ Takes in name/array data similar to arrayplotdata.  Unlike arrayplotdata,
        this never adds values to the self.arrays.  Unlike arrayplotdata, this is not
        willing to accept new entries.  It is up to the Plot object pass in the correct
        data.
        """
        if not self.writable:
            return None

        if isinstance(new_data, list) or isinstance(new_data, tuple):
            new_data = np.array(new_data) #Convert to array data

        event = {}
        
        ### If entry is in data frame, change it.  Doesn not allow for addition 
        ### to mimic behavior of dataframe (eg user should change dataframe and replot).

        try:
            self.arrays[str(name)]=(name, len(self.extras)+1, new_data)  #Overwrite whole entry
            event['changed']=[str(name)]
        except KeyError:
            raise KeyError('%s does not exist in PandasPlotData dataframe.  Add to extras if you \
            do not wish to modify the dataframe.'%name)
                   
        self.data_changed = event       
        return name
    
    def set_extra(self, name, new_data):
        """ Works like arrayplotdata.  Lets the user set extra data, which can be changed or added,
        unlike set_data which maintains the data in the dataframe at all times.
        """
        if not self.writable:
            return None

        if isinstance(new_data, list) or isinstance(new_data, tuple):
            new_data = np.array(new_data) #Convert to array data

        event = {}
        ### If entry is in data frame, change it
        try:
            self.arrays[str(name)][2]=new_data
            event['changed']=[str(name)]
        except KeyError:
            self.extras[str(name)]=(name, len(self.extras)+1, new_data) 	    
            event['added'] = [str(name)]  

        self.data_changed = event       
        return name    
    
    def update_dataframe(self, dataframe):
        ''' Wrapper for set_data to take in an entire dataframe.'''
        for column in np.asarray(dataframe.columns):
            self.set_data(column, np.asarray(dataframe[column]) )
            
    def update_extras(self, dataframe):
        ''' Wrapper for set_data to take in an entire dataframe.'''
        for column in np.asarray(dataframe.columns):
            self.set_extra(column, np.asarray(dataframe[column]))    


    #### Deprecated method that updates the dataframe in place.  For usecases, it is better to
    #### just rerun this object than overwrite the whole dataframe.
    #def set_dataframe(self, dataframe):
        #''' Completely reset plotdata with a new object.  This removes ALL 
        #old data, does not cross check for overlapping keys.'''
        #### Remove old columns of dataframe ###
        #removed={'removed':self.arrays.keys()}
        #### Add new entries
        #self._set_arrays(dataframe)
        #added={'added':self.arrays.keys()}        
        #event=dict(added.items() + removed.items() )
        #self.data_changed = event
               
    
    def _set_arrays(self, dataframe):
        ''' Sets the array dictionary, preserving sort order.'''
        return OrderedDict( (str(name),(name, idx, np.asarray(dataframe[name]) ))\
                               for idx, name in enumerate(np.asarray(dataframe.columns) ) )
        
    def _get_indicies(self, *names):
        ''' Takes in a list of names and returns indicies corresponding to.  Useful for label
        mappers.'''
        return [self.idxget(str(name)) for name in names]

    ######## These are used by chaco to inform the plot that a certain region of data is selected

    def get_selection(self, name):
        """ Returns the selection for the given column name """
        return self.selections.get(name, None)

    def set_selection(self, name, selection):
        # Store the selection in a separate dict mapping name to its
        # selection array
        self.selections[name] = selection
        self.data_changed = True



import argparse
import shlex
import logging
import os
import os.path as op

from gwu_reporter import Reporter

import logging
logger = logging.getLogger(__name__)
from skspec.logger import logclass, configure_logger

SCRIPTNAME = __name__.split('.')[-1]
SEP_DEFAULT = '/' #Not used atm

@logclass(log_name=__name__, public_lvl='debug')
def main(args=None):
    
    if args:
        if isinstance(args, basestring):
            args=shlex.split(args)
        sys.argv = args               
    
    
    parser = argparse.ArgumentParser(SCRIPTNAME, description='GWU skspec composite' 
            ' latex report assembly script', epilog='Additional help not found', 
            usage='%s --globals <command> --cmdopts' % SCRIPTNAME)    
        
    parser.add_argument('-v', '--verbosity', help='Set screen logging; '
                         'defaults to info.', nargs='?',
                         default='warning', const='info', metavar='')    
    parser.add_argument('-o', '--overwrite', action='store_true')

#    parser.add_argument('-s', '--sep', metavar='',
#                         help='Latex section separator.  Defaults to %s.  '
#                         'Set to None to avoid auto sub-sectioning' %SEP_DEFAULT)

    # Sub Commands
    subparsers = parser.add_subparsers(title='commands', metavar='', dest='op')    
    
    # TEMPLATE SUBCOMMAND
    p_template = subparsers.add_parser('template', 
        usage = '%s --globals <template> outpath --opts ' % SCRIPTNAME,    
        help='template help',
        epilog='PUT EXAMPLES HERE')
    p_template.add_argument('outpath', metavar='outfile',
        help='Name of outfile (eg foo.tex or path/to/foo.tex)')

    p_template.add_argument('-s', '--sections', metavar='', nargs='*', help='List of sections to add.'
        'or tree file.  If not tree file, default section template will be "template/section" '
        'section names will auto-break into subsections based on value of --sep',
        default=[]) 
    p_template.add_argument('-t', '--template_file', metavar='')
    
    
    
    # BUILD SUBCOMMAND
    p_build = subparsers.add_parser('build', help='Adds latex doc header including ' 
        'begin/close document, title, author etc...',
         usage = '%s --globals <build> texfile outpath --opts ' % SCRIPTNAME,    
         epilog='Put examples here')
    p_build.add_argument('texfile', help='Output file of %s template' % SCRIPTNAME)
    p_build.add_argument('outpath', metavar='outfile',
        help='Name of outfile (eg foo.tex or path/to/foo.tex)')
    
    p_build.add_argument('--title', default='Untitled', metavar='', help='Title '
                         ' in outfile')
    p_build.add_argument('--author', default='Adam Hughes', metavar='', 
                         help='Author in outfile')
    p_build.add_argument('--email', default='hugadams@gwmail.gwu.edu', metavar='', 
                         help='Email in outfile')
            

    # Compile and open
    p_build.add_argument('--nocompile', action='store_true', help='Do not compile outfile'
                         ' (eg latex outfile.tex)' ) 
    p_build.add_argument('--nopdf', action='store_true', help='When compiling,' 
                         ' will call "latex" instead of "pdflatex"')
    p_build.add_argument('-c', '--clean', action = 'store_true', help='Remove .aux, .log'
        ' and other latex temporary files.')

    
    #SUMMARY SUBCOMMAND (not implemented)
    #p_summary = subparsers.add_parser('summary', help='summary help')
    #p_summary.add_argument('texfile')

    ns = parser.parse_args()
        
    # No logfile implemented
    configure_logger(screen_level='debug', name=__name__)
    logger.info('Namespace is: %s' % ns)
    
    reporter = Reporter(**ns.__dict__) # Is this ok?
    
    if ns.op == 'template':
        reporter.output_template(ns.outpath)
    
    elif ns.op == 'build':
        reporter.make_report(ns.texfile, ns.outpath)

        # If compiling, choose pdflatex or latex
        if not ns.nocompile:
         
            if not ns.nopdf:
                latex_type = 'pdflatex'

            else:
                latex_type = 'latex'
            
            # Run latex or pdflatex twice to ensure contents load    
            os.system('%s %s' % (latex_type, ns.outpath) )
            os.system('%s %s' % (latex_type, ns.outpath) )

            
        # Cleanup latex .toc/.log/.aux files of outfile
        if ns.clean:
            outdir, outname = op.split(op.abspath(ns.outpath))
            outname = op.splitext(outname)[0]
            for ext in ['toc', 'log', 'aux', 'out']:
                wastefile = op.join(outdir, outname + '.' + ext)
                if op.isfile(wastefile):
                    logger.info('Cleaning up file: "%s"' % wastefile)
                    os.remove(wastefile)
    
    else:
        NotImplemented

if __name__ == '__main__':
    main()


from skspec.units.abcunits import ConversionUnit, UnitError
from datetime import timedelta
from pandas import DatetimeIndex, date_range, Index
import datetime
import numpy as np

class DatetimeCanonicalError(UnitError):
   """ Very specific error USED FOR CATCHING EXCEPTIONS IN TIME INDEX 
   DO NOT CHANGE THE NAME."""

class DateTime(ConversionUnit):
   """ Stores start and stop metadata.  Made for compaitiblity with interval
   unit systems. """

   symbol = 't'
   category = 'time'
   short = 'dti' #So it automatically added to intvlindex rather than Null
   full = 'timestamp'   

   
   # Not static method because requires state access (cumsum)
   def to_canonical(self, x):
      """ """
      raise DatetimeCanonicalError('DateTime unit conversion handled through'
                      ' TimeIndex because state info is required.')  

   def from_canonical(self, x):
      """   """
      raise DatetimeCanonicalError('DateTime unit conversion handled through'
                      ' TimeIndex because state info is required.')      


class IntvlUnit(ConversionUnit):
   """ Interval of time (s, min, hr), to be used in conjunction with
   DatetimeIndex in TimeSpectra.  Timespectra handles the logic of converting
   between Datetime representation and Interval representation."""

   symbol = r'$\delta t$'
   category = 'time' #Should still be time I think
   

class TimeDelta(IntvlUnit):
   """ TimeDelta; does not transform to seconds etc... because that requires
   access to original index.  This is here for completeness, but all transforms
   done through TimeSpectra.
   """
   short = 'intvl'
   full = 'interval'

   @staticmethod   
   def to_canonical(x):
      """ Convert datetimeindex to index to seconds.  Build for backwards
      compat with IntvlIndex.from_datetime() class method."""
      if hasattr(x, '__iter__'):
         return np.array([v.total_seconds() for v in x], dtype='float64')
      else:
         return x.total_seconds()

   @staticmethod      
   def from_canonical(x):
      """ This will return purely seconds, so an originally formatted HMS
      timedelta like (0,10,30), or 10min 30sec, after going to_canconical
      and back to from_canonical will be (0,0,630) IE 630 seconds.
      """
      if hasattr(x, '__iter__'):
         return np.array([datetime.timedelta(seconds=v) for v in x])
      else:
         return datetime.timedelta(seconds=x)


class Seconds(IntvlUnit):
   short = 's'
   full = 'seconds'
   _canonical = True #Or timedelta?

   @staticmethod   
   def to_canonical(x):
      return x 

   @staticmethod      
   def from_canonical(x):
      return x


class Minutes(IntvlUnit):
   short = 'm'
   full = 'minutes'

   @staticmethod   
   def to_canonical(x):
      return x * 60.0

   @staticmethod      
   def from_canonical(x):
      return x / 60.0

class Nanoseconds(IntvlUnit):
   short = 'ns'
   full = 'nanoseconds'

   @staticmethod   
   def to_canonical(x):
      return x * 10**-9

   @staticmethod      
   def from_canonical(x):
      return x / 10**-9


class Microseconds(IntvlUnit):
   short = 'us'
   full = 'microseconds'

   @staticmethod   
   def to_canonical(x):
      return x * 10**-6

   @staticmethod      
   def from_canonical(x):
      return x / 10**-6

class Milliseconds(IntvlUnit):
   short = 'ms'
   full = 'milliseconds'

   @staticmethod   
   def to_canonical(x):
      return x * 10**-3

   @staticmethod      
   def from_canonical(x):
      return x / 10**-3

class Hours(IntvlUnit):

   short = 'h'
   full = 'hours'

   @staticmethod   
   def to_canonical(x):
      return x * 3600.0

   @staticmethod      
   def from_canonical(x):
      return x / 3600.0


class Days(IntvlUnit):

   short = 'd'
   full = 'days'

   @staticmethod   
   def to_canonical(x):
      return x * 86400.0

   @staticmethod      
   def from_canonical(x):
      return x / 86400.0


class Years(IntvlUnit):

   short = 'y'
   full = 'years'

   @staticmethod   
   def to_canonical(x):
      return x * 31536000.0

   @staticmethod      
   def from_canonical(x):
      return x / 31536000.0


_intvlunits = (
   DateTime(),
   Nanoseconds(),
   Microseconds(),
   Milliseconds(),
   Seconds(),
   Minutes(),
   Hours(),
   Days(),
   TimeDelta()
)

INTVLUNITS = dict((obj.short, obj) for obj in _intvlunits)

print INTVLUNITS.keys()


from abcunits import ConversionUnit

class SoluteUnit(ConversionUnit):
   """ Goes through molar.  Test case """
   
class Moles(SoluteUnit):
   short = 'M'
   full = 'moles'
   symbol = 'M'
   _canonical = True
   
class Millimoles(SoluteUnit):
   short = 'mM'
   full = 'millimoles'
   symbol = 'mM' #Kind of redundant
   

 ### FOR TESTING
_soluteunits = (
               Moles(), 
               Millimoles(),
               ConversionUnit() #For null case
               ) 

SOLUTEUNITS = dict((obj.short, obj) for obj in _soluteunits)



# Return various templates directly as strings

import os.path as op

def opd(basename):
    ''' Allows one to open datafile using module path directory instead
        of working directory.'''
    return open(op.join(op.dirname(__file__), basename), 'r').read()


# These are returned as strings (due to .read() method above)
SIMPLE_M = opd('simple.m')





from abcindex import ConversionFloat64Index
from skspec.units import SPECUNITS

class SpecIndex(ConversionFloat64Index):
    """ """
    unitdict = SPECUNITS   
 

if __name__ == '__main__':
    import numpy as np
    x = SpecIndex(np.linspace(0,50), unit='nm')

    print x
    print x.convert('m')
    print x.convert('cm')
    print x.convert('um')   
    print x.convert('f')
    print x.convert('ev')
    print x.convert('w')
    print x.convert(None)


""" Various utilities used for manipultating custom spectral dataframe objects.
    Some of these may work for any dataframe, but others require special attributes
    such as "baseline" and "filedict".  For serialization operations, import 
    from spec_serial.py 
    """

from pandas import Series, DataFrame
import numpy as np
from scipy import integrate
from types import GeneratorType

from skspec.pandas_utils.dataframeserial import _get_metadict
from skspec.exceptions import badvalue_error
import skspec.config as pvconfig

import logging
logger = logging.getLogger(__name__) 


class UtilsError(Exception):
    """ """

def _compute_span(index, with_unit=False):
   """ Format the first, last entries in an index for string output
   like (start - final).  Doesn't include the unit
   """
   imin, imax = index.min(), index.max()
   try:
      span = '%.2f - %.2f' % (imin, imax)

   except TypeError:
      try: #hack for datetimeindex
         span = '%s - %s' % (
               str(imin).split()[1], #Cut out year
               str(imax).split()[1]
           )
         # Just string format if all else fails
      except Exception:
         span = '%s - %s' % (imin, imax)

   if with_unit:
      span = '%s %s' % (span, index._unit.short)
   return span


def hasgetattr(obj, attr, default=None):
    """ Combines hasattr/getattr to return a default if hasattr fail."""
    if not hasattr(obj, attr):
        return default
    return getattr(obj, attr)


def safe_lookup(obj, attr, missing=pvconfig.MISSING):
    """ Look up attribute on object.  If not found, or set to None,
    will return a missing value defaulting to pvconfig.MISSING.
    """
    return hasgetattr(obj, attr, missing)


def countNaN(obj):
    ''' Returns counts of nans in an object'''
    return np.isnan(obj.sum()).sum()   

def _parse_generator(generator, astype=tuple):
    """ Convert generator as tuple, list, dict or generator.
        
    Parameters
    ----------
    astype : container type (tuple, list, dict) or None
        Return expression as tuple, list... if None, return as generator. 

    Notes
    -----
    Mostly useful for operations that in some cases return a dictionary, 
    but also might be useful as a list of kv pairs etc...
    """        
    if not isinstance(generator, GeneratorType):
        raise UtilsError("Generator required; got %s" % type(generator))
    
    if isinstance(astype, str):
        astype = eval(astype)        

    if astype:
        return astype(generator)

    else:
        return generator   

### Rather deprecated due to TimeSpectra.reference/iunit
def divby(df, divisor=0, axis=0, sameshape=True):
    ''' Small wrapper around pandas.divide() that takes in either column integers, names,
    or entire arrays as the divisor.  Note, pandas.divide by default is rowwise, but here
    it defaults to column-wise.
    
    Parameters:
    -----------
      df:  Dataframe object.
      divisor: (See notes below)
      axis: Division axis. 0=column-wise division. 1=row-wise
      sameshape: If true, function will check that the divisor as the same shape as
        the data along that axis.  For example, is an array of 2000 units, 
        and axis=0, then the row dimension must be 2000 for the dataframe.
        AFAIK df.div does not enforce this stringency, which sometimes can lead
        to hard-to-find bugs.
    
    Notes:
    ------
      This program first checks if divisor is an integer.  If not, it attempts
      to lookup by rowname or columnname.  Finally, it tries to divide by the divisor itself,
      if the divisor is an array/series.  This seemed like a flexible way to do this 
      without too much type checking.'''

    ### Column-wise division
    if axis==0:        
        if isinstance(divisor, int):
            divisor=df.columns[divisor]
        
        else:
            try:
                divisor=df[divisor]
            except Exception:
                pass #divisor=divisor
            
        divlen=df.shape[0] #Store length of a single column


    ### Rowwise division    
    elif axis==1:
        if isinstance(divisor, int):
            divisor=df.ix[divisor]
        
        else:
            try:
                divisor=df.xs(divisor)
            except KeyError:
                pass
            
        divlen=df.shape[1]
        
    else:
        raise badvalue_error('axis', '0,1')
    
    ### Enforce strict shapetype
    if sameshape:
        if len(divisor) != divlen:
            raise TypeError('Divisor dimensions %s do not match dividend' 
            ' dimension %s along axis = %s'%(divlen, len(divisor), axis))
    
    return df.divide(divisor, axis=axis)
    
            

def boxcar(df, binwidth, axis=0):
    ''' Only works on boxcar for now.  Also, want binwidth and/or binnumber
    to be inputs but not both. '''
    if axis==1:
        binnumber=len(df.columns)/binwidth    
        counts, binarray=np.histogram(df.columns, bins=binnumber)
        digiarray=np.digitize(np.asarray(df.columns, dtype=float), binarray)          
        mapped_series=Series(digiarray, index=df.columns)       
        dfout=df.groupby(mapped_series, axis=1).mean()
        dfout.columns=binarray
    
    elif axis==0:
        binnumber=len(df.index)/binwidth  #Converted to int when np.histogram called
        counts, binarray=np.histogram(df.index, bins=binnumber)
        digiarray=np.digitize(np.asarray(df.index, dtype=float), binarray)    
        
        mapped_series=Series(digiarray, index=df.index)
        
        dfout=df.groupby(mapped_series, axis=0).mean()
        dfout.index=binarray   
        
    else:
        raise AttributeError('Axis must be 0 (column binning) or 1 (index/row) binning.  You entered %s'%axis)
    
    return dfout

                    
                                

def digitize_by(df, digitized_bins, binarray, axis=0, avg_fcn='mean', weight_max=None):
    ''' Takes in an array of digitized bins, and then restructures a dataframe
    based on the bin array'''
    
    ### Map the digitized bins to series data with axis same as one that will be collapsed ###
    if axis == 0:
        mapped_series=Series(digitized_bins, index=df.index) #No, this isn't wrong, just weird
    else:
        #mapped_series=Series(digitized_bins, index=df.columns)
        mapped_series=Series(digitized_bins, index=df.index)
    
    if len(df.shape)==1:
        if avg_fcn.lower() == 'mean':
            dfout=df.groupby(mapped_series).mean()  #// is importanmt
    
        elif avg_fcn.lower() == 'sum':
            dfout=df.groupby(mapped_series).sum()
    
        ### Rebins according to the sum, and then divides axis or rows by their maxes.
        ### If I want a normalized array, can call this with bindwidth=1.0
        elif avg_fcn.lower() == 'weighted':
            dfout=df.groupby(mapped_series).mean()
            if weight_max:
                dfout= dfout.apply(lambda x: x / weight_max)  #These should be float division
            else:
                dfout= dfout.apply(lambda x: x/ x.max())
            
        else:
            raise NotImplementedError('%s is not a valid key to df_rebin, must \
                                     be mean, sum or weighted'%avg_fcn)        
    
    elif len(df.shape)==2:
        if avg_fcn.lower() == 'mean':
            dfout=df.groupby(mapped_series, axis=axis).mean()
            dfout.index=binarray
            
    
        elif avg_fcn.lower() == 'sum':
            dfout=df.groupby(mapped_series, axis=axis).sum()
    
        ### Rebins according to the sum, and then divides axis or rows by their maxes.
        ### If I want a normalized array, can call this with bindwidth=1.0
        elif avg_fcn.lower() == 'weighted':
            dfout=df.groupby(mapped_series, axis=axis).mean()
            if weight_max:                
                dfout=dfout.apply(lambda x:x / weight_max, axis=axis)
            else:
                dfout=dfout.apply(lambda x: x / x.max(), axis=axis)            
            
        else:
            raise NotImplementedError('%s is not a valid key to df_rebin, must \
                                     be mean, sum or weighted'%avg_fcn)

    else:
        raise NotImplementedError('df_rebin only works with 1-d or 2-d arrays')        
        
    return dfout   


def split_by(df, n, axis=1, astype=list):
    """ Returns a list of dataframes sampled by n.  For example, a datframe
    of 20 columns, sampled by n=4, will return 5 evenly spaced dataframes.
    If uneven samples, will raise warning.  axis=1 for column sampling, axis=0
    for row sampling.
    """
    
    if n > df.shape[1]:
        raise UtilsError("Requested samplesize (n=%s) exceeds size along axis %s"
                         % (n,axis))
    
    # Integery guarantees rounding down
    m = int(float(df.shape[axis]) / n)
    df_out=[]
    for i in range(n):
        if i==n-1:
            df_out.append(df[df.columns[i*m::]])
        else:
            df_out.append(df[df.columns[i*m:(i+1)*m]])

    if n % df.shape[axis]:
        logger.warn("Returning uneven sampling for (n=%s) along axis %s"
                    "of size %s" % (n, axis, df.shape[axis]))
                    
    return df_out     
    

def rebin(df, binwidth, axis=0, avg_fcn='weighted', weight_max=None):
    ''' Pass in an array, this slices and averages it along some spacing increment (bins).
    Axis=0 means averages are computed along row.  axis=1 means averages are computed along column.
    Dataframe already handles most issues, such as if binning in unequal and/or binning is larger than 
    actual length of data along axis.  Aka bin 100 rows by 200 rows/bin.
    Binwidth is the spacing as in every X entries, take an average.  Width of 3 would be
    every 3 entries, take an average.
    
    Redundant because if series is passed in, the axis keyword causes errors.
    
    If using avg_fcn='weighted', one can pass an upper limmit into the "weight_max" category
    so that weighting is to a fixed value and not to the max of the dataset.  This is
    useful when comparing datasets objectively.  For dataframes, weight_max may be a 1d
    array of the normalization constant to each column in the dataframe.  If a single value
    is entered, or for a series, that value will be divided through to every row.
    
    Note: avg_fct='weighted' and weight_max=None will find a max after binning the data, 
    and divide all other column(or row values) by the max.  
    This is not the statistical normaization, which should be added later (X-u / sigma).'''


    if len(df.shape)==1:
        if avg_fcn.lower() == 'mean':
            dfout=df.groupby(lambda x:x//binwidth).mean()  #// is importanmt
    
        elif avg_fcn.lower() == 'sum':
            dfout=df.groupby(lambda x:x//binwidth).sum()
    
        ### Rebins according to the sum, and then divides axis or rows by their maxes.
        ### If I want a normalized array, can call this with bindwidth=1.0
        elif avg_fcn.lower() == 'weighted':
            dfout=df.groupby(lambda x:x//binwidth).mean()  #Groupby uses int division
            if weight_max:
                dfout=dfout.apply(lambda x: x / weight_max)      #Apply uses float division
            else:
                dfout=dfout.apply(lambda x: x/ x.max())
            
        else:
            raise NotImplementedError('%s is not a valid key to df_rebin, must \
                                     be mean, sum or weighted'%avg_fcn)        
    
    elif len(df.shape)==2:
        if avg_fcn.lower() == 'mean':
            dfout=df.groupby(lambda x:x//binwidth, axis=axis).mean()
    
        elif avg_fcn.lower() == 'sum':
            dfout=df.groupby(lambda x:x//binwidth, axis=axis).sum()
    
        ### Rebins according to the sum, and then divides axis or rows by their maxes.
        ### If I want a normalized array, can call this with bindwidth=1.0
        elif avg_fcn.lower() == 'weighted':
            dfout=df.groupby(lambda x:x//binwidth, axis=axis).mean()
            if weight_max:                
                dfout=dfout.apply(lambda x:x / weight_max, axis=axis)
            else:
                dfout=dfout.apply(lambda x: x / x.max(), axis=axis)            
            
        else:
            raise NotImplementedError('%s is not a valid key to df_rebin, must \
                                     be mean, sum or weighted'%avg_fcn)

    else:
        raise NotImplementedError('df_rebin only works with 1-d or 2-d arrays')        
        
    return dfout   

def maxmin_xy(obj, style='max', arg=False, idx=True, val=True):
    ''' Return arg (eg integer index position), index val, and object val
        at the data maximum.  Works on series or dataframes.
        
        Params:
        -------
          obj: 1 or 2-d array type.
          style: return 'max' or 'min' vals along the dataset.
          arg: Index position at maximum.
          idx: Index val at maximum.
          val: Return the data val at this point.'''
    
    if obj.ndim == 1:
        return _maxmin_xy(obj, style=style, arg=arg, idx=idx, val=val)

    elif obj.ndim == 2:
        return obj.apply(_maxmin_xy, style=style, arg=arg, idx=idx, val=val)

    else:
        raise badval_error('ndim', '1,2')
        

def _maxmin_xy(array, style='max', arg=False, idx=True, val=True):
    ''' Return index and val at maximum in a series.  For index position (eg 6) at max,
        use argmax()'''    
    
    if style not in ['max', 'min']:
        raise badval_error('style', 'max, min')        
    
    if not arg and not idx and not val:
        raise AttributeError('max_xy requires at least one of the following boolean arguments: arg, idx, val')    
    
    out=[]    

    if style=='max':               #X: idxmax/idxmin no longger array attributes
        fidx=array.argmax(); farg=array[array.idxmax()]; fval=array.max() 
    else:
        fidx=array.argmin(); farg=array[array.idxmin()]; fval=array.min()
        
    
    if arg:
        out.append(farg)

    if idx:
        out.append(fidx)

    if val:
        out.append(fval)
    
    if len(out) > 1:
        out=tuple(out)
    else:
        out=out[0]
    
    return out

def find_nearest(obj, value, arg=False, idx=True, val=True):
    if obj.ndim == 1:
        return _find_nearest(obj, value=value, arg=arg, idx=idx, val=val)  #Positional args still passed as kwargs

    elif obj.ndim == 2:
        return obj.apply(_find_nearest, value=value, arg=arg, idx=idx, val=val)

    else:
        raise badval_error('ndim', '1,2')    
    
    
def _find_nearest(array, value, arg=False, idx=True, val=True):
    '''Find nearest val in an array, return index and array val.
      Parameters:
      ---------
         array: Actually think this is set up for series/dataframe?

      Notes:
      ------
         This works by computing the absolute difference array of the value and looking
         for minimums along this array as seen in stack overflow discussions.
         http://stackoverflow.com/questions/2566412/find-nearest-value-in-numpy-array
    '''

    ### Get index and arg from diff array
    diff_array=np.abs(array-value)
    arg, idx=_maxmin_xy(diff_array, style='min', arg=arg, idx=idx, val=val)
    
    ### Find corresponding value from original array
    val=array[idx]
    
    out=[]
    if arg:
        out.append(arg)
        
    if idx:
        out.append(idx)
        
    if val:
        out.append(val)
    
    if len(out) > 1:
        out=tuple(out)
    else:
        out=out[0]
    
    return out    


from abcunits import ConversionUnit


KFACTOR = 273.15 #Difference Kelvin, C (how precise is this known?)

   
class TempUnit(ConversionUnit):
   """ Temperature units.  ALl conversions go through Kelvin. """

#http://www.metric-conversions.org/temperature/fahrenheit-to-kelvin.htm
class Kelvin(TempUnit):
   short = 'K'
   full = 'Kelvin' #Proper names, keep this way?
   symbol = 'r$^{\deg}K$' #Isn't degree Kelvin technially wrong?
   _canonical = True
   
   @staticmethod
   def to_canonical(x):
      return x
      
   @staticmethod
   def from_canonical(x):
      return x

class Celsius(TempUnit):
   short = 'C'
   full = 'Celsius'
   symbol = 'r$^{\deg}C$'
   
   @staticmethod
   def to_canonical(x):
      return x + KFACTOR
      
   @staticmethod
   def from_canonical(x):
      return x - KFACTOR


class Farenheiht(TempUnit):
   short = 'F'
   full = 'Farenheiht'
   symbol = 'r$^{\deg}F$'
   
   @staticmethod
   def to_canonical(x):
      return ((x - 32.00)/1.80) + 273.15
      
   @staticmethod
   def from_canonical(x):
      return ((x - KFACTOR) * 1.80) + 32.00


_tempunits = (Kelvin(),
              Celsius(),
              Farenheiht(),
              ConversionUnit() #For null case
              )
TEMPUNITS = dict((obj.short, obj) for obj in _tempunits)


''' Model for building correct parameters that will be assigned to timespectra
    in gwu_spec script.'''
from skspec.exceptions import ParameterError, badkey_check
from skspec.core.spectra import _normdic
import os.path as op

import logging
logger = logging.getLogger(__name__)

def _to_bool(attr, value):
    ''' Convert a value to bool.  If str is "true/false/none", converts.
        Useful since users may want to set norm=None from cmdline, and argparse
        will yiled norm="None" for example.'''

    if isinstance(value, basestring):
        if value.lower() == 'none':
            value = False
        elif value.lower() == 'true':
            value = True
        elif value.lower() == 'false':
            value = False
            
        else:
            raise ParameterError('Parameter must be true, false or None: %s'
                                 ' received %s' % value)            
    return bool(value)


class Parameters(object):
    ''' Defines timespectra scripting parameters. '''

    reference_default = 0
    specunit_default = 'nm'
    intvlunit_default = 'm'
    uv_ranges_default = 8
    # or((430.0,450.0), (450.0,515.0), (515.0, 570.0),
    #(570.0,620.0), (620.0,680.0))
    
    t_start_default = None
    t_end_default = None 
    xmin_default = None
    xmax_default = None

    norm_default = [None, 'r', 'a']
    sub_base_default = False
    
    valid_minmax_default = None #nm
    
    bline_fit_default = False
    fit_regions_default = None
    
    git = True

    
    def __init__(self, **params):
        ''' Set defaults from either parameters or class defaults. '''
        
        self._params = params
        
        self.reference = self.loud_apply('reference', self.reference_default)
        try:
            self.reference = int(self.reference)
        except Exception:
            pass
        

        if 'git' in params:
            self.git = False
        
        self.specunit = self.loud_apply('specunit', self.specunit_default)
        self.intvlunit = self.loud_apply('intvlunit', self.intvlunit_default)
        if self.intvlunit.lower() == 'none':
            self.intvlunit=None
        self.uv_ranges = self.loud_apply('uv_ranges', self.uv_ranges_default)
        
        self.t_start = self.loud_apply('t_start', self.t_start_default)
        self.t_end = self.loud_apply('t_end', self.t_end_default)

        # Slicing wavelengths (not sure how to handle times)
        self.x_min = self.loud_apply('x_min', self.xmin_default)
        self.x_max = self.loud_apply('x_max', self.xmax_default)
        
        # Float-convert slices (should be properties)
        if isinstance(self.x_min, basestring):
            if self.x_min.lower() == 'none':
                self.x_min = None
                
        if isinstance(self.x_max, basestring):
            if self.x_max.lower() == 'none':
                self.x_max = None                
                
        if self.x_min is not None:                    
            self.x_min = float(self.x_min)
        
        if self.x_max is not None:
            self.x_max = float(self.x_max)
            
        # Boolean defaults
        self.sub_base = self.loud_apply('sub_base', self.sub_base_default, boolean=True)       
        self.bline_fit = self.loud_apply('bline_fit', self.bline_fit_default, boolean=True)

        # Properties (don't use loud_apply)
        self.norms = self._params.pop('norms', self.norm_default)
        self.fit_regions = self._params.pop('fit_regions', self.fit_regions_default)
        self.valid_minmax = self._params.pop('valid_minmax',
                                    self.valid_minmax_default)
        
        # Warn if unused self._params
        if self._params:
            name = self.__class__.__name__
            logger.critical("Unused parameters passed to %s: %s" % 
                        (name, self._params.keys()))
            
    @property
    def norms(self):
        return self._norms
        
    @norms.setter
    def norms(self, norms):
        ''' Returns list of valid units (None, 'a', 'r') '''
        if not norms:
            logger.warn('"norms" parameter not found.  Setting to [None].  Only'
                        ' rawdata will be analyzed.')
            logger.debug("Converting 'norms' from str to list")            
            self._norms = [None]         
            return
        
        if not hasattr(norms, '__iter__'):
            logger.debug("Converting 'norms' list")
            norms = norms.split()
                
        # Make sure all outtypes (a, r, None) are valid; convert "None" to none
        for idx, otype in enumerate(norms):
            if isinstance(otype, basestring):
                if otype.lower() == 'none':
                    logger.debug('Changing inuit="None" (str) to Nonetype')
                    norms[idx] = None                    
            badkey_check(norms[idx], _normdic.keys())
 
        self._norms = norms         

        
    @property
    def fit_regions(self):
        return self._fit_regions
    
    # Probably best nto to set these through cmdline, as it's hard to ensure
    # proper type, but if must, this calls eval
    @fit_regions.setter
    def fit_regions(self, regions):
        if regions is None:
            self._fit_regions = regions
            return 
        
        if isinstance(regions, basestring):
            regions = eval(regions)
        if not hasattr(regions, '__iter__'):
            raise ParameterError('fit_regions must be an iterable of pairs: '
                                 'received %s' % (regions))
        self._fit_regions = regions

    @property
    def valid_minmax(self):
        return self._valid_minmax        
        
    @valid_minmax.setter
    def valid_minmax(self, minmax):
        if minmax is None:
            self._valid_minmax = None
            return 
        
        if isinstance(minmax, basestring):
            minmax = eval(minmax)
            
        if not hasattr(minmax, '__iter__'):
            raise ParameterError('fit_regions must be an iterable of pairs: '
                                 'received %s' % (minmax))
        
        if len(minmax) != 2:
            raise ParameterError('valid_minmax must be lenth two iterable IE '
                    '(560, 600).  Received: %s' % minmax)
  
        self._valid_minmax = minmax

    def loud_apply(self, attr, default, boolean=False):
        
        if self._params.has_key(attr):
            outval = self._params[attr]
            self._params.pop(attr)

        else:
            logger.debug('%s not found in user-supplied parameters. '  
            'Automatically setting to "%s"' % (attr, default))
            outval = default        

        if boolean:
            outval = _to_bool(attr, outval)
        
        return outval
    
    # Useful for methods calling params.items() (implement full dict interface later)
    def items(self):
        return [(attr, getattr(self, attr)) for attr in self.__dict__ 
                if attr != '_params']
    
    
    def as_markdownlist(self):
        """ Return parameters for markdown list for .ipynb use:
            - foo = bar
            - baz = ing
        """

        out = ''
        for k,v in sorted(self.items()):            
            out += ('\"- **%s:** %s\\n\",\n' % (k,v))
            
        # Pain in my ass here...
        out = out.lstrip('"')
        out = out.rstrip()
        out = out.rstrip(',')
        out = out.rstrip('"')
        return out
            

class USB2000(Parameters):
    ''' Define some shared parameters between ocean optics spectrometers.'''
    
    # Change spectral parameter defaults in NM
    xmin_default = 430.0 
    xmax_default = 680.0
    valid_minmax_default = (339.0, 1024.0) #nm

    sub_base_default = True
    bline_fit_default = True
    fit_regions_default = ((345.0, 395.0), (900.0, 1000.0))
    
        
class USB650(Parameters):
    
    valid_minmax_default = (200.0, 850.0) #nm
    sub_base_default = True
    bline_fit_default = False
    
    # Set xmin/xmax?
        


''' File IO and miscellaneous utilities'''

__author__ = "Adam Hughes"
__copyright__ = "Copyright 2012, GWU Physics"
__license__ = "Free BSD"
__maintainer__ = "Adam Hughes"
__email__ = "hugadams@gwmail.gwu.edu"
__status__ = "Development"

import os
from pandas import DataFrame, read_csv, concat
from skspec.core.imk_utils import get_files_in_dir, get_shortname


def df_from_directory(directory, csvargs, sortnames=False, concat_axis=1, shortname=True, cut_extension=False):
    ''' Takes files from a directory, presuming they are identically formatted, and reads them into
        a dataframe by iterating over read_csv().  All arguments that read_csv() would take are passed
        in.  These kwargs are in regard the files themselves, for example, skiprows, skipfooter and header.
        
        For now, no support for glob to take only files of a certain file extension.
        For now, conctaentation occurs along

    Args:
       directory- Path to directory where files are stored.
       csvargs- Dictionary of arguments that are passed directly to the read_csv() function. For example
                skiprows, header, na_values etc... see pandas API for read_csv()
        
    Kwds:
       sortnames- Will attempt to autosort the filelist. Otherwise, files are ordered by the module
                  os.path.listdir().
                  
       concat_axis- How to merge datafiles into a dataframe.  Default is axis=1, which means all files 
                    should share the same index values.  I use this for spectral data where the wavelength
                    column is my dataframe index.
                  
       shortname- If false, full file path is used as the column name.  If true, only the filename is used. 
       
       cut_extension- If kwd shortname is True, this will determine if the file extension is saved or cut from the data.
       '''

    filelist=get_files_in_dir(directory)

    if shortname:
        fget=lambda x:get_shortname(x, cut_extension=cut_extension)
    else:
        fget=lambda x: x
    
    ### Either full names or short names of filelist    
    working_names=[fget(afile) for afile in filelist]       

    ### This parameter can't be passed in, so if user does so, pull it out.
    try:
        csvargs.pop('names')
    except KeyError:
        pass
    else:
        raise Warning('Please do not enter a names keyword for csvargs, it gets inferred from the filenames in\
        the directory.')

    dflist=[read_csv(afile, names=[fget(afile)], **csvargs) for afile in filelist]
    
    ### THIS IS BUSTED, PUTTING NANS EVERYWHERE EXCEPT ONE FILE, but dflist itself ws nice.
    dataframe=concat(dflist, axis=1)
                        
    ### concat tries to sort these, so this will preserve the sort order
    if sortnames:
        dataframe=dataframe.reindex(columns=sorted(working_names))

    return dataframe
            

if __name__=='__main__':
    datadirectory='./NPConcentration'
    
    read_csv_args={'sep':',', 'header':2, 'index_col':0, 'skiprows':2, 'na_values':' \r'}
  
    df=df_from_directory(datadirectory, read_csv_args, sortnames=True)




import re
import skspec
import skspec.data
import matplotlib.pyplot as plt

from datetime import datetime
from collections import OrderedDict
from skspec import Spectra


from IPython.html.widgets import (
            FlexBox, VBox, HBox, HTML, Box, RadioButtons,
            FloatText, Dropdown, Checkbox, Image,
            IntSlider, Button, Text, FloatSlider, IntText, ContainerWidget
                                  )
from IPython.utils.traitlets import (
            link, Unicode, Float, Int, Enum, Bool, Instance, Any
                                     )


from specgui import Box, HTML
from nbtools import mpl2html, log_message

from skspec.core.spectra import _normdic as NUdic
import skspec.config as pvconf
from skspec.data import aunps_glass
from skspec.plotting.advanced_plots import PLOTPARSER

class SpectraModel(HTML, Box):
    """
    A notional "complex widget" that knows how to redraw itself when key
    properties change.
    """
    
    # CONSTANTS (These are not traits)
    classname = Unicode("btn btn-success",sync=True)
    title = Unicode("Popover Test",sync=True)
    CONTENT = Unicode("""Lovely popover :D. Color in green using class btn btn-success""",sync=True)
    html = Bool(sync=True)
    
    DONT_DRAW = re.compile(r'^(_.+|value|keys|comm|children|visible|parent|log|config|msg_throttle)$')
    NORMUNITS = NUdic
    NORMUNITS_REV = OrderedDict((v,k) for k,v in NORMUNITS.items())
    COLORS = ["b","g","r","y","k"]
    COLORMAPS = sorted(m for m in plt.cm.datad if not m.endswith("_r"))
    SLIDER_STEPS = Float(25)
    
    # IO traitlets
    load_spec = Bool(False,sync=True)
    load_file = Bool(True,sync=True) #
    file_name = Unicode("<Variable>", sync=True)
    save_spec = Bool(False,sync=True)
    save_spec_as = Unicode('Test',sync=True)
    
    # Spectra traits
    spec = Instance(Spectra)
    testdataset = Unicode('<Dataset>', sync=True) 
    spec_modified = Instance(Spectra)
    
    # Plotting Traits
    figwidth = Float(6.5)
    figheight = Float(6.5)
    interactive = Bool(False, sync=True)
    colorbar = Bool(False, sync=True)
    autoupdate = Bool(True, sync=True)
    colormap = Enum(COLORMAPS ,sync=True)
    color = Enum(COLORS, default_value = 'k', sync=True)
    kind = Enum(PLOTPARSER.keys(), default_value = 'spec', sync=True)
    
    # Units
    spec_unit = Unicode
    var_unit = Unicode
    iunit = Unicode
    norm_unit = Enum(NORMUNITS.values(),  sync=True)
    
    # Message/warnings
    message = Unicode
    
    # Sampling/slicing
    #specslice_axis = Enum([0,1], default_value=0, sync=True)
    specslice_position_start = Float(sync=True)
    specslice_position_end = Float(sync=True)
    specslider_start = Float(sync=True)
    specslider_end = Float(sync=True)
    specstep = Float(sync=True)
    specspacing = Int(1, sync=True)
    
    timeslice_position_start = Float(sync=True)
    timeslice_position_end = Float(sync=True)
    timeslider_start = Float(sync=True)
    timeslider_end = Float(sync=True)
    timestep = Float(sync=True)
    timespacing = Int(1, sync=True)
    
    selectlines = Bool(False, sync=True)
    
    # User Defined Function
    user_f = Unicode(sync=True)
    
    
    def __init__(self, *args, **kwargs):
        
        # Initialize traits (_spec_changed calls initial draw)
        super(SpectraModel, self).__init__(*args, **kwargs)
        self._dom_classes += ("col-xs-9",)
    
    
    # DEFAULTS
    # --------
    def _spec_default(self):
        return getattr(skspec.data, 'aunps_water')()
    
    def _colormap_default(self):
        return pvconf.CMAP_1DSPECPLOT #Use skspec config default (red/blue map)
    
    # Events
    # ------
    def _spec_changed(self, name, old, new):
        """Overall spectrum changes; triggers most events."""
        
        # Leave this at this position in loop
        self.spec_modified= self.spec
        # --------------
        
        self._FREEZE = True #pause draws/slicing
        
        # Units
        self.spec_unit = self.spec.full_specunit
        self.var_unit = self.spec.full_varunit
        self.norm_unit = self.spec.full_norm
        self.iunit = self.spec.full_iunit
        
        
        # Spec slicing
        self.specslice_position_start = self.spec.index[0]
        self.specslice_position_end = self.spec.index[-1]
        self.specslider_start = self.spec.index[0]
        self.specslider_end = self.spec.index[-1]
        self.specstep = (self.spec.index.max() - self.spec.index.min())/self.SLIDER_STEPS
        self.specspacing = 1
        
        self.timeslice_position_start = self.spec.columns[0]
        self.timeslice_position_end = self.spec.columns[-1]
        self.timeslider_start = self.spec.columns[0]
        self.timeslider_end = self.spec.columns[-1]
        self.timestep = 10#(self.spec.columns.max() - self.spec.columns.min())/self.SLIDER_STEPS
        self.timespacing = 1
        
        # Plot defaults to color map
        self._color_state = False
        
        self._FREEZE = False
        self.draw(name, old, new)
    
    def _norm_unit_changed(self, name, old, new):
        self.spec_modified = self.spec_modified.as_norm(self.NORMUNITS_REV[new])
        self.draw(name, old, new)
    
    def _iunit_changed(self, name, old, new):
        self.spec_modified.iunit = new
        self.draw(name, old, new)
    
    # Plotting events
    # ---------------    
    def _figwidth_changed(self, name, old, new):
        self.draw(name, old, new)

    def _figheight_changed(self, name, old, new):
        self.draw(name, old, new)
    
    def _colormap_changed(self, name, old, new):
        self._color_state = False
        self.draw(name, old, new)
    
    def _color_changed(self):
        """ Because this sets colorbar, might cause 2 redraws,
            so _FREEZE used to prevent this
            """
        self._FREEZE = True
        self.colorbar = False
        self._color_state = True
        self._FREEZE = False
        self.draw()
    
    def _colorbar_changed(self, name, old, new):
        self._color_state = False
        self.draw(name, old, new)
    
    def _interactive_changed(self, name, old, new):
        self.draw(name, old, new)
    
    # This should be phased out; plots should support colormap, area should handle accordingly
    def _kind_changed(self, name, old, new):
        self.draw(name, old, new)
    
    def _selectlines_changed(self, name, old, new):
        if self.interactive:
            self.draw(name, old, new)
    
    # IO Events
    # ---------
    # THIS SHOULD BE LOAD BUTTON CLICKED!!!!
    def _file_name_changed(self):
        try:
            self.spec = getattr(skspec.data, self.file_name)()
        except AttributeError:
            pass

    @log_message    
    def save_to_ns(self):
        get_ipython().user_ns[self.save_spec_as]=self.spec_modified
    
    @log_message
    def load_from_ns(self, var):
        self.spec = get_ipython().user_ns[var]            
        
    # Slicing events
    # --------------
    def _specslice_position_start_changed(self, name, old, new):       
        if not self._FREEZE:
            self.slice_spectrum(name)
            self.draw(name, old, new)
    
    def _specslice_position_end_changed(self, name, old, new):
        if not self._FREEZE:
            self.slice_spectrum(name)
            self.draw(name, old, new)
    
    def _timeslice_position_start_changed(self, name, old, new):
        if not self._FREEZE:
            self.slice_spectrum(name)
            self.draw(name, old, new)
    
    def _timeslice_position_end_changed(self, name, old, new):
        if not self._FREEZE:
            self.slice_spectrum(name)
            self.draw(name, old, new)
    
    
    def _timespacing_changed(self, name, old, new):
        """ Don't let user set less than 1 or more than dataset size"""
        # Will have to update when add var/spec slicing
        #axis = self.slice_axis
        if self.timespacing < 1:
            self.timespacing = 1
        elif self.timespacing > self.spec_modified.shape[1]:
            self.timespacing = self.spec_modified.shape[1]
        
        self.slice_spectrum(name)
        self.draw(name, old, new)
    
    def _specspacing_changed(self, name, old, new):
        """ Don't let user set less than 1 or more than dataset size"""
        # Will have to update when add var/spec slicing
        #axis = self.slice_axis
        if self.specspacing < 0:
            self.specspacing = 0
        elif self.specspacing > self.spec_modified.shape[0]:
            self.specspacing = self.spec_modified.shape[0]
        
        self.slice_spectrum(name)
        self.draw(name, old, new)
        
    # Draw/Slice updates
    # ------------------
    @log_message
    def slice_spectrum(self, name=None):
        """ Slice and resample spectra """
        self.spec_modified = self.spec.nearby[self.specslice_position_start:self.specslice_position_end:self.specspacing,
                                              self.timeslice_position_start:self.timeslice_position_end:self.timespacing]
    
    @log_message
    def apply_userf(self,name=None):
        import numpy as np
        self.spec_modified = self.spec_modified.apply(eval(self.user_f))
        self.draw(name)
    
    
    
    @log_message
    def draw(self, name=None, old=None, new=None):
        if name is not None and self.DONT_DRAW.match(name):
            return
        
        if self._FREEZE:
            return
        
        plot_and_message = ''
        
        # Better way would be a decorator or something that only goes into draw if not autoupdate
        if self.autoupdate:
            
            # Generate new figure object
            f = plt.figure(figsize=(self.figwidth, self.figheight))
            if PLOTPARSER.is_3d(self.kind):
                projection = '3d'
            else:
                projection = None
            ax = f.add_subplot(111, projection=projection)
            
            if self._color_state or self.kind not in ['spec', 'waterfall', 'contour', 'contour3d']:
                colorkwags = dict(color=self.color)
            else:
                colorkwags = dict(cmap=self.colormap, cbar=self.colorbar)
            
            self.spec_modified.plot(ax=ax,
                                    fig=f,
                                    kind=self.kind,
                                    norm=self.NORMUNITS_REV[self.norm_unit],
                                    **colorkwags
                                    )
            f.tight_layout() #Padding around plot
            lines = ax.get_lines()
            plt.close(f)
                                    
            #http://mpld3.github.io/modules/API.html
            if self.interactive:
                import mpld3
                if self.selectlines:
                    from line_plugin import HighlightLines
                    
                    for idx, col in enumerate(self.spec_modified.columns):
                        name = 'COLUMN(%s): %s' % (idx, col)
                        tooltip = mpld3.plugins.LineLabelTooltip(lines[idx], name)
                        #voffset=10, hoffset=10,  css=css)
                        mpld3.plugins.connect(f, tooltip)
                    
                    mpld3.plugins.connect(f, HighlightLines(lines))
                
                plot_and_message += mpld3.fig_to_html(f)
            else:
                plot_and_message += mpl2html(f)
            
            self.fig_old = f
        
        else:
            plot_and_message += html_figure(self.fig_old)
        
        # VALUE IS WHAT GUI LOOKS UP!!!
        self.value = plot_and_message



import time

def mpl2html(fig, title="Figure"):
    """ Converts a matplotlib fig into an IPython HTML Core object for display 
    compatibility in widgets."""
    from IPython.core.pylabtools import print_figure
    import base64

    fdata64 = base64.b64encode(print_figure(fig))
    html_tpl = '<img alt="{title}" src="data:image/png;base64,{fdata64}">'
    return html_tpl.format(**locals())


def log_message(f):
    def wrapper(*args, **kwargs):
        """ args[0] is Spectrogram; args[1] is name of calling function"""

        # Some methods like gui button clicks don't pass args right
        try:
            caller = args[1]
        except IndexError:
            caller = ''
        
        tstart = time.time()
        args[0].message = '<div class="alert alert-warning"> %s triggered by' \
                ' %s </div>' % (f.__name__, caller)
        try:
            out = f(*args, **kwargs)
        except Exception as exc:
            args[0].message = '<div class="alert alert-danger"> %s Failed with' \
                ' exception: (%s) </div>' % (caller, exc.message)
            raise exc
        else:
            args[0].message = '<div class="alert alert-success"> %s() complete' \
                ' (%.2f sec) </div>' % (f.__name__, time.time()-tstart)
        
        return out
    return wrapper


''' Loader utilities to easily import example data either from directory, or
    directly from pickle/csv files. '''

import os
import sys

from pkgutil import get_loader
from pandas import read_csv

from skspec.pandas_utils.dataframeserial import df_load
import skspec.exceptions as errors

DATAPATH='example_data/'
PACKAGE='skspec'

def _get_data_smart(package, resource, as_stream=True):
    """Rewrite of pkgutil.get_data() that actually lets the user determine if data should
    be returned read into memory (aka as_stream=True) or just return the file path.
    """

    loader = get_loader(package)
    if not hasattr(loader, 'get_data'):
        return
    
    mod = sys.modules.get(package) or loader.load_module(package)
    if not hasattr(mod, '__file__'):
        return

    # Modify the resource name to be compatible with the loader.get_data
    # signature - an os.path format "filename" starting with the dirname of
    # the package's __file__
    parts = resource.split('/')
    parts.insert(0, os.path.dirname(mod.__file__))
    resource_name = os.path.join(*parts)

    if as_stream:
        return loader.get_data(resource_name)
    else:
        return resource_name
    

def get_exampledata(filename='spectra.csv', as_stream=False):
    '''Import skspec example data.  Similar to pkgutil.get_data without forcing
       read into memory.
       
       Parameters
       ----------
    
       filename :  str
           File or directory name in skspec/example_data to import.
    
       as_stream : 
           If true, file is read into memory using open(filename, 'rb').read().  
           Otherwise, filepath is returned as a string.'''

    
    filepath=os.path.join(DATAPATH, filename)
    return _get_data_smart(PACKAGE, filepath, as_stream)


def get_csvdataframe(filename='spectra.csv', **csvargs):
    ''' Takes in spectral data, either .pickle or .csv, returns dataframe.  
        Files must be in same path as  called by get_exampledata.  If csv file,
        one can pass csvargs throught the **csvargs dictionary.'''

    filepath=get_exampledata(filename, as_stream=False)
    ext=os.path.splitext(filepath)[1]

    if ext == '.csv':
        return read_csv(filepath, **csvargs)

    # Is this deprecated/useful?
    elif ext == '.pickle':
        return df_load(filepath)

    else:
        raise errors.GeneralError('%s must have file extension .csv or .pickle, not '
                             '%s' %(filename, ext))
        
        
    
if __name__=='__main__':
    data=get_csvdataframe(filename='spectra.pickle')  



""" Plots for correlation analysis. Designed for SPEC2D OBJECTS """

import numpy as np
from skspec.plotting.advanced_plots import _gen2d3d, add_projection
import matplotlib.pyplot as plt
import skspec.plotting.plot_utils as pvutil
import matplotlib.colorbar as mplcbar
import matplotlib.ticker as mplticker
from mpl_toolkits.mplot3d import Axes3D, axes3d, art3d #Need Axes3d for 3d projection!

class CorrPlotError(Exception):
    """ """

def _corr2d4fig(spec, a1_label=r'$\bar{A}(\nu_1)$', 
               a2_label=r'$\bar{A}(\nu_2)$', **contourkwds): 
    """ Abstract layout for 2d correlation analysis plot.  
    
    **contourkwds
        Passed directly to _gencontour; includes keywords like xlabel, ylabel
        and so forth.
    """

    # Maybe this should take X, Y, Z not ts

    #fig, ax #how to handle these in general 2d
    # Maybe it's helpful to have args for top plots (ie ax1,2,3)
    
    title = contourkwds.pop('title', '')
    cbar = contourkwds.pop('cbar', False)
    grid = contourkwds.setdefault('grid', True) #Adds grid to plot and side plots
    
    # REFACTOR THIS
    cbar_nticks = 5
  
    
    # This will create a fig
    ax1 = plt.subplot2grid((5,5), (0,0), colspan=1) # top left
    plt.subplots_adjust(hspace = 0, wspace=0)    # Remove whitespace
    
    ax1.plot([0,-1], color='black')    

    ax1.text(.18, -.78, a1_label, size=12) 
    ax1.text(.55, -.35, a2_label, size=12)    

    ax2 = plt.subplot2grid((5,5), (0,1), colspan=4) # top
    ax3 = plt.subplot2grid((5,5), (1,0), colspan=1, rowspan=4) #left
    ax4 = plt.subplot2grid((5,5), (1, 1), colspan=4, rowspan=4) #main contour
    ax3.invert_xaxis()
    ax4.yaxis.tick_right()
    ax4.xaxis.tick_bottom() #remove top xticks
    ax4.yaxis.set_label_position('right')
    
    ax4, contours = _gen2d3d(spec, ax=ax4, **contourkwds)
        
    # Bisecting line
    pvutil.diag_line(ax4)  
    
    # Fig is created by _gen2d in ax4 _gen2d3d
    fig = plt.gcf()
      
    # Hide axis labels 
    for ax in [ax2, ax3]:
        if grid:
            pvutil.hide_axis(ax, axis='both', axislabel=True, ticklabels=True)
        else:
            pvutil.hide_axis(ax, axis='both', hide_everything = True)
            
    pvutil.hide_axis(ax1, axis='both', hide_everything=True)
  
    #plt.colorbar() doesn't work
    # Handles its own colorbar (See links below; important)
   # http://stackoverflow.com/questions/13784201/matplotlib-2-subplots-1-colorbar
   # http://matplotlib.org/api/colorbar_api.html#matplotlib.colorbar.make_axes
    if cbar:
        if cbar in ['left', 'right', 'top', 'bottom']:
        # if bottom or right, should repad this
            location = cbar
        else:
            location = 'top'
        cax,kw = mplcbar.make_axes([ax1, ax2, ax3, ax4], 
                                   location=location,
                                   pad = 0.05,
                                   aspect = 30, #make skinnier
                                   shrink=0.75) 
        
        cb = fig.colorbar(contours, cax=cax,**kw)# ticks=[0,zz.max().max()], **kw)
        cb.locator = mplticker.MaxNLocator(nbins=cbar_nticks+1) #Cuts off one usually
        cb.set_label(spec.iunit)        
        cb.update_ticks()


    #ax1 will take care of itself in contour
    if grid:
        if grid == True:
            ax2.grid()
            ax3.grid()
  
        else:
            ax2.grid(color=grid)
            ax3.grid(color=grid)

    fig.suptitle(title, fontsize='large') # Still overpads
    return (ax1, ax2, ax3, ax4)



def corr2d(spec, sideplots='mean', **pltkwargs):
    """ Visualize synchronous, asynchronous or phase angle spectra.

    Parameters
    ----------
    
    sideplots: str or bool ('mean')
        If True, sideplots will be put on side axis of cross plots.  Use
        'empty' to return blank sideplots.  mean', 'min', 'max', will 
        plot these respective spectra on the sideplots.


    fill : bool (True)
        Contours are lines, or filled regions.

    **pltkwargs: dict
        Any valid matplotlib contour plot keyword, or skspec general
        plotting keyword (grid, title etc...)

    Returns
    -------

    tuple (matplotlib.Axes)
        If side plots, returns (ax1, ax2, ax3, ax4)
        If not side plots, returns ax4 only

    """
 
    SIDEPAD = 1.10
 
    _reverse_axis = pltkwargs.pop('_reverse_axis', False)

    #Side plot line kwds (passed to Spectrum())
    linekwds = dict(linewidth=1, 
                    linestyle='-', 
                    color='k',  #black
                    custompadding=0,
                    )
       
    if sideplots:
        if sideplots == True:
            sideplots = 'mean'

        # Should be identical, but maybe not when do heterspectral corr...
        try:
            symbol1 = spec.index._unit.symbol
            symbol2 = spec.columns._unit.symbol
        except Exception:
            symbol1 = symbol2 = r'\lambda'
                
        if spec._corr2d.center is not None:
            label1 = r'$\bar{A}(%s_1)$' % symbol1
            label2 = r'$\bar{A}(%s_2)$' % symbol2

        else:
            label1, label2 = r'$A(%s_1)$' % symbol1, r'$A(%s_2)$' % symbol2

        # BASED ON SELF, NOT BASED ON THE OBJECT ITSELF!
        ax1, ax2, ax3, ax4 = _corr2d4fig(spec, label1, label2, **pltkwargs )

        # Side plots    
        data_orig_top = spec._corr2d.spec.loc[spec.index[0]:spec.index[-1], :]
        data_orig_side = spec._corr2d.spec.loc[spec.columns[0]:spec.columns[-1], :]

        top =  None

        if sideplots == 'mean':
            top = data_orig_top.mean(axis=1)
            side = data_orig_side.mean(axis=1)

        elif sideplots == 'max':
            top = data_orig_top.max(axis=1)
            side = data_orig_side.max(axis=1)
            
        elif sideplots == 'min':
            top = data_orig_top.min(axis=1)
            side = data_orig_side.min(axis=1)
            
        elif sideplots == 'all':
            top = data_orig_top
            side = data_orig_side

        elif sideplots == 'empty':
            pass

        else:
            raise CorrPlotError('sideplots keyword must be "mean", "max", "min",'
                         ' "all", or "empty".')

        if top is not None:
            top.plot(ax=ax2, **linekwds)
            side.plot(ax=ax3, **linekwds) 
            for ax in (ax2, ax3):
                pvutil.hide_axis(ax, axis='both')
                ax.set_title('')

        # Reorient ax3
        pvutil.invert_ax(ax3)

        #Set sideplot labels    
        if sideplots != 'empty':
            ax2.set_ylabel(sideplots)
            ax2.yaxis.set_label_position('right')
            
        #Scale x/y padding just a hair 
        y1, y2 = ax2.get_ylim()
        ax2.set_ylim(y1, SIDEPAD*y2)

        x1, x2 = ax3.get_xlim()
        ax3.set_xlim(SIDEPAD*x1, x2)
        
        if _reverse_axis:
            ax4.set_ylim(ax4.get_ylim()[::-1]) 
            ax4.set_xlim(ax4.get_xlim()[::-1]) 
            
            #sideplots
            ax2.set_xlim(ax2.get_xlim()[::-1])
            ax3.set_ylim(ax3.get_ylim()[::-1]) 

        return (ax1, ax2, ax3, ax4)

    else:
        # If no sideplots, can allow for 3d plots
        pltkwargs.setdefault('kind', 'contour')
        ax = pltkwargs.pop('ax', _gen2d3d(spec, **pltkwargs)[0]) #return axes, not contours
        if _reverse_axis:
            ax.set_ylim(ax.get_ylim()[::-1]) 
            ax.set_xlim(ax.get_xlim()[::-1]) 
        return ax

    
def corr3d(spec, projection='xy', **pltkwargs):
    """ pltkwargs are passed to wire plot.  Special keyword 'contourkwargs'
    can pass arguments directly to contour plot.
    """

    contourkwargs = pltkwargs.pop('contourkwargs', {})

    # Like this view better
    pltkwargs.setdefault('elev', 45)
    pltkwargs.setdefault('azim', -135)            
        
    ax = spec.plot(kind='wire', **pltkwargs)    

    # if fill, will will draw over 3d (bug)
    if projection:
        # Pass contourkwargs, not plotkwards
        ax = add_projection(spec, ax=ax, plane=projection, **contourkwargs) 
    
    return ax


# How to mix multiplot:
#http://matplotlib.org/examples/mplot3d/mixed_subplots_demo.html
def corr_multi(corr2d, **pltkwargs):
    """ """
    
    sync = corr2d.sync #Used to look up some span attributes for now
    
    # Boilerplate multiplot

    _title_default = '%s (%s)  scale: %s' \
        % (corr2d.spec.name, sync._var_span, corr2d._scale_string)
    
    title = pltkwargs.pop('title', _title_default)
    grid = pltkwargs.pop('grid', True)
    tight_layout = pltkwargs.pop('tight_layout', False)
    figsize = pltkwargs.pop('figsize', (8,8))

    f, (ax1, ax2, ax3, ax4) = pvutil.splot(2,2, fig=True, figsize=figsize)
    f.suptitle(title) #, fontsize=20)
    if tight_layout:
        f.tight_layout()
        
    if 'cbar' in pltkwargs:
        raise NotImplementedError('Colorbar not supported.')
        
 
    ax1 = corr2d.dyn_spec.plot(ax=ax1,
                           title='Dynamic Spectra',
                           color='k')    


    pltkwargs['sideplots'] = None #Necessary
    pltkwargs['kind'] = 'corr2d'


    ax2 = corr2d.async_codist.plot(ax=ax2,
                                   title='Async. Codistribution ($\Delta$)', 
                                   **pltkwargs)

    ax3 = sync.plot(ax=ax3, 
                           title='Sync. Correlation ($\Phi$)',
                            **pltkwargs)

    ax4 = corr2d.async.plot(ax=ax4,
                            title='Async. Correlation ($\Psi$)',
                            **pltkwargs)


    # Hide axis labels
    for ax in (ax1, ax2, ax3, ax4):
        if grid:
            pvutil.hide_axis(ax, axis='both', axislabel=True, ticklabels=True)
        else:
            pvutil.hide_axis(ax, axis='both', hide_everything = True)    

    # Soft diagonal line
    for ax in (ax2, ax3, ax4):
        pvutil.diag_line(ax)     

    return (ax1, ax2, ax3, ax4)
    